import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.font.FontRenderContext;
import java.lang.*;
import java.awt.geom.Ellipse2D;

public class BloqueGrafico extends JPanel implements Runnable, DQDB_Observer, MouseListener, MouseMotionListener
{
	private boolean mensajesGraficosActivados;
	private boolean terminadoresCerrados;
	private boolean animacionEnCurso;
	private int ultimoEvento;
	private boolean[] visibilidad;
	private int[] eventosActivos;
	private int[] eventosActuales;
	private boolean animarRecepcionDeCeldas;
	private boolean modoEspera;
	private int estacionComentada;
	private Image comentario, comentarioCA1, comentarioCA2, comentarioContadorPeticionesCA1,comentarioContadorPeticionesCA2, comentarioContador, comentarioContadorVacias, comentarioContadorPeticiones;
	private Datos datosComentarioUsuario;
	private int xComentario, yComentario, comentarioCeldaOrigen, comentarioCeldaDestino;
	private Estacion[] estaciones;
	private int xAnimacion;
	private Vector<CeldaGrafica> celdas1, celdas2;
	private boolean borrarEventosAntiguos;
	private int estacionMonitorizadaAudio, estacionMonitorizadaTexto;
	private FactoriaMultimedia factoria;
	private BloqueSimulacion bloqueSimulacion;
	private InterfazGrafica interfaz;
	public static int VER_TODAS=1, VER_SELECCIONADAS=2, OCULTAR_TODAS=3, OCULTAR_SELECCIONADAS=4;
	private int modoVisibilidad; //-1=desactivado, 1=seleccion de seleccionadas, 2=ocultar seleccionadas
	private int version;
	private Image[] imagenVersion;
	private int numeroEstaciones;
	private Image[] imagenesEstacion;
	private Image imagenPeticionInsertada, imagenCeldaOcupada, imagenCeldaVacia;
	private Image imagenCanal, imagenUsuario, imagenTerminadorIzquierdo, imagenTerminadorDerecho, imagenTerminadorIzquierdoAbierto, imagenTerminadorDerechoAbierto, marcadorAudio, marcadorTexto, marcadorAudioTexto, txPendienteCA1, txPendienteCA2, txPendiente_V3V4, peticionPendiente, peticionPendienteGhost, CA1InsertaDatos, CA2InsertaDatos;
	private int altoEstacion, anchoEstacion, anchoCanal, anchoTerminador;
	int margenX, margenY, margenMinimoX;
	private JViewport viewport;
	private int idEventoActual;
	private CargadorDeImagenes cargador;
	
	//CONSTRUCTOR
	public BloqueGrafico(BloqueSimulacion bloqueSimulacion, InterfazGrafica interfaz, int version, int numeroEstaciones,  double pgUnica, double[] arrayPg, boolean borrarEventosAntiguos, boolean animarRecepcionDeCeldas)
	{
		super();
		cargador=CargadorDeImagenes.getCargador();
		
		mensajesGraficosActivados=Constantes.activarMensajesGraficos;
		terminadoresCerrados=Constantes.ocultarInteriorTerminadores;
		visibilidad=new boolean[numeroEstaciones];
		eventosActivos=new int[numeroEstaciones];
		eventosActuales=new int[numeroEstaciones];
		for(int i=0; i<visibilidad.length; i++) visibilidad[i]=true;
		modoEspera=false;
		comentarioCeldaOrigen=-1; 
		comentarioCeldaDestino=-1;
		estaciones=bloqueSimulacion.getEstaciones();
		celdas1=new Vector<CeldaGrafica>();
		celdas2=new Vector<CeldaGrafica>();
		this.borrarEventosAntiguos=borrarEventosAntiguos;
		this.animarRecepcionDeCeldas=animarRecepcionDeCeldas;
		
		
		estacionMonitorizadaAudio=-1;
		estacionMonitorizadaTexto=-1;
		this.interfaz=interfaz;
		factoria=FactoriaMultimedia.getFactoria();
		this.bloqueSimulacion=bloqueSimulacion;

		imagenPeticionInsertada=cargador.getImagen("peticionInsertada.gif");
		
		txPendienteCA1=cargador.getImagen("CA1TxPendiente.gif");
		txPendienteCA2=cargador.getImagen("CA2TxPendiente.gif");
		txPendiente_V3V4=cargador.getImagen("TxPendienteV34.gif");
		peticionPendiente=cargador.getImagen("peticionPendiente.gif");
		peticionPendienteGhost=cargador.getImagen("peticionPendienteGhost.png");
		imagenCeldaOcupada=cargador.getImagen("celdaOcupada.gif");
		imagenCeldaVacia=cargador.getImagen("celdaVacia.gif");
		imagenUsuario=cargador.getImagen("usuario.gif");
		CA1InsertaDatos=cargador.getImagen("CA1InsertaDatos.gif");
		CA2InsertaDatos=cargador.getImagen("CA2InsertaDatos.gif");
		
		setCursor(FactoriaMultimedia.cursorDefault);
		modoVisibilidad=-1;

		imagenVersion=new Image[4];
		for(int i=0; i<imagenVersion.length; i++)
			imagenVersion[i]=cargador.getImagen("version"+(i+1)+".gif");
			
		this.numeroEstaciones=numeroEstaciones;
		this.version=version;

		
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
			
		setBackground(Color.white);			
			
		imagenTerminadorIzquierdo=cargador.getImagen("terminadorIzquierdo.gif");
		imagenTerminadorDerecho=cargador.getImagen("terminadorDerecho.gif");
		imagenTerminadorIzquierdoAbierto=cargador.getImagen("terminadorIzquierdoAbierto.gif");
		imagenTerminadorDerechoAbierto=cargador.getImagen("terminadorDerechoAbierto.gif");
		imagenCanal=cargador.getImagen("canal.gif");
		
		marcadorAudio=cargador.getImagen("marcadorAudio.gif");
		marcadorTexto=cargador.getImagen("marcadorTexto.gif");
		marcadorAudioTexto=cargador.getImagen("marcadorAudioTexto.gif");
		
		imagenesEstacion=new Image[2];
		imagenesEstacion[0]=cargador.getImagen("estacionCerrada.gif");
		imagenesEstacion[1]=cargador.getImagen("estacionAbiertaVersion"+version+".gif");
			
		altoEstacion=imagenesEstacion[0].getHeight(null);
		anchoTerminador=imagenTerminadorIzquierdo.getWidth(null);
		anchoEstacion=imagenesEstacion[0].getWidth(null);
		anchoCanal=imagenCanal.getWidth(null);
		
		margenMinimoX=10;
		margenX=margenMinimoX;
		margenY=60;
		
		
		addMouseListener(this);
		addMouseMotionListener(this);
			
		Dimension dim=new Dimension(2*margenX+2*anchoTerminador+(numeroEstaciones+1)*anchoCanal+numeroEstaciones*anchoEstacion, 2*margenY+altoEstacion);
		setPreferredSize(dim);	
	}
	
	
	//Mtodo qe permite mostrar u ocultar el interior de los terminadores
	public void setOcultarTerminadores(boolean b)
	{
		terminadoresCerrados=b;
		repaint();
	}
	
	//Mtodo que permite activar o desactivar la animacin de celdas
	public void setAnimarRecepcionDeCeldas(boolean b)
	{
		animarRecepcionDeCeldas=b;
	}
	
	
	//Mtodo que establece el viewport que controla la vista de este bloque
	public void setViewport(JViewport viewport)
	{
		this.viewport=viewport;
	}
	
	//Mtodo que desactiva el modo de seleccin activo
	public void anularSeleccionActiva()
	{
		setCursor(FactoriaMultimedia.cursorDefault); //Esto anula el modo seleccion de texto y audio puesto qe dependen del cursor que haya seleccionado, al ponerlo por defecto ya no se puede seleccionar estacin
		modoVisibilidad=-1; //Esto anula los modos de seleccin de estaciones a mostrar u ocultar
	}

	//Mtodo que permite cambiar la Pg de una estacin del bloque simulacin
	public void cambiarPg(int numeroEstacion, double nuevaPg)
	{
		bloqueSimulacion.cambiarPg(numeroEstacion, nuevaPg);
	}
	
	//actualiza el tamao del bloque al tamao adecuado al nmero de estaciones
	public void redimensionar()
	{ 
		Dimension dim=new Dimension(2*margenX+2*anchoTerminador+(numeroEstaciones+1)*anchoCanal+numeroEstaciones*anchoEstacion, 2*margenY+altoEstacion);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);
		
		viewport.revalidate();
		viewport.repaint();
	}
	
	
	//Mtodo paintComponent sobreescrito para realizar las tareas de este BloqueGrafico
	public void paintComponent(Graphics g)
	{
		if (g instanceof Graphics2D)
		{
      		Graphics2D g2 = (Graphics2D)g;
      		Map<Object, Object> rMap = new HashMap<Object, Object>();
        	Object key = RenderingHints.KEY_ANTIALIASING;
        	Object value = RenderingHints.VALUE_ANTIALIAS_ON;
        	rMap.put(key, value);
        	g2.addRenderingHints(rMap);
    	}
		super.paintComponent(g);
		
		
		if(estacionMonitorizadaAudio!=-1 && estacionMonitorizadaAudio==estacionMonitorizadaTexto) g.drawImage(marcadorAudioTexto, margenX+anchoTerminador+anchoCanal+(anchoEstacion+anchoCanal)*estacionMonitorizadaAudio+15, 1, this);
		else
		{
			if(estacionMonitorizadaAudio!=-1) g.drawImage(marcadorAudio, margenX+anchoTerminador+anchoCanal+(anchoEstacion+anchoCanal)*estacionMonitorizadaAudio+60, 1, this);
			if(estacionMonitorizadaTexto!=-1) g.drawImage(marcadorTexto, margenX+anchoTerminador+anchoCanal+(anchoEstacion+anchoCanal)*estacionMonitorizadaTexto+60, 1, this);
		}
		
		
		Point point=viewport.getViewPosition();
		Rectangle rec=viewport.getViewRect();
		
		
		//Centra el dibujo en caso de ser menor que el area de pantalla
		if (viewport!=null) 
		{
			int xMaxima=2*margenX+2*anchoTerminador+(numeroEstaciones+1)*anchoCanal+numeroEstaciones*anchoEstacion;

			if (rec.getWidth()>xMaxima) 
			{
				margenX+=(rec.getWidth()-xMaxima)/2;
			}
			else if (rec.getWidth()<xMaxima) margenX=margenMinimoX;
		}
		int xActual=margenX;
		int yActual=margenY;
		
	
		xActual+=anchoTerminador;
		g.drawImage(imagenCanal,xActual,yActual,this);
		xActual+=anchoCanal;
		
			
		//Optimiza el pintado: solo pinta las estaciones visibles;
		int xInicio=(int) point.getX();
		int indiceInicio=0;
		int bloque=anchoEstacion+anchoCanal;
		if(xInicio>=margenX+anchoTerminador+anchoCanal)
		{
			xInicio=xInicio-margenX-anchoTerminador-anchoCanal;
			indiceInicio=xInicio/bloque-1;
			if(indiceInicio<0) indiceInicio=0;
		}
		
		int xFinal=(int) (point.getX()+rec.getWidth());
		int indiceFinal=0;
		if(xFinal>=margenX+anchoTerminador+anchoCanal)
		{
			xFinal=xFinal-margenX-anchoTerminador-anchoCanal;
			indiceFinal=xFinal/bloque+1;
			if(indiceFinal>numeroEstaciones) indiceFinal=numeroEstaciones;
		}			
		
		//Calculamos las coordenadas de inicio dela estacion correspondiente a indiceDeInicio
		xActual=margenX+anchoTerminador+anchoCanal+indiceInicio*bloque;

					
		
		Celda celda;
		
		//El siguiente bucle se encarga de pintar nicamente las estaciones correspondientes a la zona visible del panelGrafico
		//Esto supone una optimizacin importante: Para 1000 estaciones no se hace un bucle de 1000 iteraciones sino que independientemente del nmero de estaciones siempre se realiza un bucle de con tantas iteraciones como estaciones que caben en pantalla, es decir, unas pocas iteraciones
		for(int i=indiceInicio; i<indiceFinal; i++)
		{		

				if(visibilidad[i])
				{
					//Pintamos la estacion abierta
					g.drawImage(imagenesEstacion[1],xActual,yActual,this);
					
					//Pintamos el nmero de estacin
					g.setFont(new Font("Arial", Font.BOLD, 14));
					if(version==1 || version==2) g.drawString("EST."+(i+1), xActual+10,yActual-5);
					else g.drawString("EST."+(i+1), xActual+10,yActual-40);
					
					//PINTAMOS LAS SEALES ENVIADAS EN EL SLOT ACTUAL:
					
					//Obtenemos el estado de la estacin	
					Vector<Integer> listaDeEstado=estaciones[i].getEstado();
				
					//Pintamos las seales pasadas (estticas)
					boolean iconoUsuario=false;
					for(int j=0; j<listaDeEstado.size()-eventosActuales[i]; j++) 
					{
						if(listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA1 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA2 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_NO_GENERA_DATOS)
							g.drawImage(FactoriaMultimedia.getImagenEstatica(listaDeEstado.elementAt(j)), xActual, yActual+58, this);
						else iconoUsuario=true;	
							
					}
						
					//Pintamos las seales activas (animadas)
					for(int j=listaDeEstado.size()-eventosActuales[i]; j<listaDeEstado.size(); j++) 
					{
						if (j>-1)
						{
							if(listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA1 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA2 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_NO_GENERA_DATOS)
								g.drawImage(FactoriaMultimedia.getImagen(listaDeEstado.elementAt(j)), xActual, yActual+58, this);
							else iconoUsuario=true;	
						}
						
					}
						
					//Pintamos el icono de usuario
					if (iconoUsuario)
					{
						if(estaciones[i].getDatosGeneradosPorUsuario()!=null) g.drawImage(FactoriaMultimedia.getImagen(Estacion.USUARIO_GENERA_DATOS_CA1), xActual+28, yActual+87, this);
						else g.drawImage(FactoriaMultimedia.getImagen(Estacion.USUARIO_NO_GENERA_DATOS), xActual+28, yActual+87, this);	
					}
					else g.drawImage(imagenUsuario, xActual+29, yActual+87, this);	
					
					g.setFont(new Font("Arial", Font.BOLD, 10));
							
					if (version==3 || version==4)
					{
						//Pintamos los contadores de las versiones 3 y 4
						g.setFont(new Font("Arial", Font.PLAIN, 11));
						g.drawString("CD: ", xActual, yActual-25);
						g.drawString("VACIAS: ", xActual, yActual-15);
						g.drawString("PETICIONES: ", xActual, yActual-5);
						

						g.drawString("CD: ", xActual, yActual+altoEstacion+35);
						g.drawString("VACIAS: ", xActual, yActual+altoEstacion+25);
						g.drawString("PETICIONES: ", xActual, yActual+altoEstacion+15);
					
						//Pintamos los valores actuales de los contadores
						g.drawString(""+estaciones[i].getCuentaAtras(1), xActual+21, yActual-25);
						g.drawString(""+estaciones[i].getContadorVacias(1), xActual+47, yActual-15);
						g.drawString(""+estaciones[i].getContadorPeticiones(1), xActual+64, yActual-5);
						
					
						g.drawString(""+estaciones[i].getCuentaAtras(2), xActual+21, yActual+altoEstacion+35);
						g.drawString(""+estaciones[i].getContadorVacias(2), xActual+47, yActual+altoEstacion+25);
						g.drawString(""+estaciones[i].getContadorPeticiones(2), xActual+64, yActual+altoEstacion+15);
					
						//Pintamos el icono de peticiones pendientes y el de TxPendiente en los CA que las tengan
						if (estaciones[i].getDatosPendientes(1)!=null) g.drawImage(txPendiente_V3V4, xActual+47, yActual+38, this);

						if (estaciones[i].getDatosPendientes(2)!=null) g.drawImage(txPendiente_V3V4, xActual+47, yActual+156, this);
						
						if(estaciones[i].getPeticionesPendientes(1)>0) g.drawImage(peticionPendiente, xActual+5, yActual+39, this);
						else g.drawImage(peticionPendienteGhost, xActual+5, yActual+39, this);
						if(estaciones[i].getPeticionesPendientes(2)>0) g.drawImage(peticionPendiente, xActual+5, yActual+157, this);
						else g.drawImage(peticionPendienteGhost, xActual+5, yActual+157, this);
						
						//Pintamos el valor de los contadores de peticiones pendientes de envo
						g.setColor(estaciones[i].getPeticionesPendientes(1)!=0? Color.black: Color.LIGHT_GRAY);
						g.drawString("x"+estaciones[i].getPeticionesPendientes(1), xActual+19, yActual+49);
						
						g.setColor(estaciones[i].getPeticionesPendientes(2)!=0? Color.black: Color.LIGHT_GRAY);
						g.drawString("x"+estaciones[i].getPeticionesPendientes(2), xActual+19, yActual+167);
						
						g.setColor(Color.black);
					}
					
					else
					{
							if (estaciones[i].getDatosPendientes(1)!=null) g.drawImage(txPendienteCA1, xActual, yActual+12, this);		
							if (estaciones[i].getDatosPendientes(2)!=null) g.drawImage(txPendienteCA2, xActual, yActual+151, this);
				
					}
					
					if (i<celdas1.size() && celdas1.elementAt(i).getDatosRecienInsertados()) g.drawImage(CA1InsertaDatos, xActual, yActual+12, this);	
					if (celdas2.size()==numeroEstaciones) 
					{
						if(celdas2.elementAt(i).getDatosRecienInsertados()) g.drawImage(CA2InsertaDatos, xActual, yActual+151, this);			
					}
					else if (i-(numeroEstaciones-celdas2.size())>=0) 					
					{
						if(celdas2.elementAt(i-(numeroEstaciones-celdas2.size())).getDatosRecienInsertados()) g.drawImage(CA2InsertaDatos, xActual, yActual+151, this);			
					}					
				}
				
			
				g.setFont(new Font("Arial", Font.BOLD, 10));
			
				xActual+=anchoEstacion;
				g.drawImage(imagenCanal,xActual,yActual,this);

				
				xActual+=anchoCanal;
		}
		
		
		
		//Calculamos las coordenadas de la estacin correspondiente  esta iteracin
		xActual=margenX+anchoTerminador+anchoCanal+numeroEstaciones*bloque;
		
		
		
		int indiceInicio2=Math.min(indiceInicio, 0);
		int indiceFinal2=Math.min(indiceFinal+1, celdas1.size());
		Datos datosCelda;
		for(int i=indiceInicio2; i<indiceFinal2; i++)
		{
			if (i<celdas1.size() && celdas1.elementAt(i).getCelda()!=null) 
			{
				datosCelda=celdas1.elementAt(i).getDatos();
				if(datosCelda!=null)
				{
				
					g.drawImage(imagenCeldaOcupada,margenX+xAnimacion+celdas1.elementAt(i).getX(), celdas1.elementAt(i).getY(), this);
					
					//Escribimos los datos de la celda centrados y suavizados
					Font fuente=new Font("Arial", Font.BOLD, 10);
				
					String texto=datosCelda.toString();
					Graphics2D graphics2d = (Graphics2D)g;
					graphics2d.setFont(fuente);
					graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
					FontRenderContext frc = graphics2d.getFontRenderContext();
					java.awt.font.LineMetrics linemetrics = fuente.getLineMetrics(texto, frc);
					int largo = (int)fuente.getStringBounds(texto, frc).getWidth();
					int desfaseX=(73-largo)/2;

					if (celdas1.size()>i) 
					{
						CeldaGrafica celdaAux=celdas1.elementAt(i);
						graphics2d.drawString(texto, margenX+xAnimacion+celdaAux.getX()+desfaseX+24, celdaAux.getY()+16);
					}
			
				}
					
				else g.drawImage(imagenCeldaVacia, margenX+xAnimacion+celdas1.elementAt(i).getX(), celdas1.elementAt(i).getY(), this);
			
				if(celdas1.elementAt(i).getPeticion()) g.drawString("P", margenX+xAnimacion+celdas1.elementAt(i).getX()+15, celdas1.elementAt(i).getY()+16);
				if(celdas1.elementAt(i).getPeticionRecienInsertada()) g.drawImage(imagenPeticionInsertada, margenX+xAnimacion+celdas1.elementAt(i).getX()+12, celdas1.elementAt(i).getY()+6, this);
			}
			
			if (i<celdas2.size() && celdas2.elementAt(i).getCelda()!=null) 
			{
				
				datosCelda=celdas2.elementAt(i).getDatos();
				if(datosCelda!=null) 
				{
					g.drawImage(imagenCeldaOcupada,margenX-xAnimacion+celdas2.elementAt(i).getX(), celdas2.elementAt(i).getY(), this);
					

					//Escribimos los datos de la celda centrados y suavizados
					Font fuente=new Font("Arial", Font.BOLD, 10);
					String texto=datosCelda.toString();
					Graphics2D graphics2d = (Graphics2D)g;
					graphics2d.setFont(fuente);
					graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
					FontRenderContext frc = graphics2d.getFontRenderContext();
					java.awt.font.LineMetrics linemetrics = fuente.getLineMetrics(texto, frc);
					int largo = (int)fuente.getStringBounds(texto, frc).getWidth();

					int desfaseX=(73-largo)/2;

					graphics2d.drawString(datosCelda.toString(), margenX-xAnimacion+celdas2.elementAt(i).getX()+desfaseX+24, celdas2.elementAt(i).getY()+16);

				}
					
				else g.drawImage(imagenCeldaVacia, margenX-xAnimacion+celdas2.elementAt(i).getX(), celdas2.elementAt(i).getY(), this);
				if(celdas2.elementAt(i).getPeticion()) g.drawString("P", margenX-xAnimacion+celdas2.elementAt(i).getX()+15, celdas2.elementAt(i).getY()+16);
				if(celdas2.elementAt(i).getPeticionRecienInsertada()) g.drawImage(imagenPeticionInsertada, margenX+xAnimacion+celdas2.elementAt(i).getX()+12, celdas2.elementAt(i).getY()+6, this);
			}
		}
		
		

		for(int i=indiceInicio; i<indiceFinal; i++)
		{
			//Si la estacin est cerrada (no visible) dibujaremos el nmero de estacin sobre ella
			if (!visibilidad[i])
			{
				//Posicion de la estacin actual;
				xActual=margenX+anchoTerminador+anchoCanal+i*bloque;
				g.drawImage(imagenesEstacion[0],xActual,yActual,this);

				//Escribimos el nmero de estacin centrado y suavizado
				Font fuente=new Font("Arial", Font.BOLD, 20);
				String texto=""+(i+1);
				Graphics2D graphics2d = (Graphics2D)g;
				graphics2d.setFont(fuente);
				graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				FontRenderContext frc = graphics2d.getFontRenderContext();
				java.awt.font.LineMetrics linemetrics = fuente.getLineMetrics(texto, frc);
				int largo = (int)fuente.getStringBounds(texto, frc).getWidth();
				int alto = (int)fuente.getStringBounds(texto, frc).getHeight();
				int desfaseX=(anchoEstacion-largo)/2;
				graphics2d.drawString(""+(i+1), xActual+desfaseX,yActual+183);	
			}				
		}	
		
		//Determinamos la posicion del terminador Izquierdo para pintarlo
		xActual=margenX+anchoTerminador+anchoCanal+numeroEstaciones*bloque;		
	
		//Dibujamos y rellenamos dos rectnculos blancos para tapar los extremos de las celdas que sobresalen de los terminadores, dado que el ancho de los terminadores es menor que el de las celdas
		g.setColor(Color.white);
		g.fillRect(margenX-anchoEstacion, margenY, anchoEstacion, altoEstacion);
		g.fillRect(xActual+anchoTerminador,yActual, anchoEstacion, altoEstacion);
		
		//Dibuja los terminadores cerrados	
		if (terminadoresCerrados)
		{
			g.drawImage(imagenTerminadorIzquierdo,margenX,margenY,this);
			g.drawImage(imagenTerminadorDerecho,xActual,yActual,this);

		}
		
		//Dibuja el interior de los terminadores
		else
		{
			g.drawImage(imagenTerminadorIzquierdoAbierto,margenX,margenY,this);
			g.drawImage(imagenTerminadorDerechoAbierto,xActual,yActual,this);	
		}
		
		//Dibuja la versin DQDB actual
		g.drawImage(imagenVersion[version-1], 10, 10, this);
		
		
		//PINTA COMENTARIOS:
		if (comentario!=null)
		{
		 	g.drawImage(comentario, xComentario, yActual+yComentario, this);
			g.setColor(Color.black);
			g.setFont(new Font("Arial", Font.BOLD, 10));
			if(comentarioCeldaOrigen!=-1) g.drawString(""+comentarioCeldaOrigen, xComentario+64, yActual+yComentario+30);
			if(comentarioCeldaOrigen!=-1) g.drawString(""+comentarioCeldaDestino, xComentario+64, yActual+yComentario+40);
		}
		
		if (datosComentarioUsuario!=null)
		{
			if (datosComentarioUsuario.getOrigen()!=-1)
			{
				g.drawImage(cargador.getImagen("msg-usuarioGenera.gif"), xComentario, yActual+yComentario, this);
				g.setFont(new Font("Arial", Font.BOLD, 13));
				g.setColor(Color.black);
				g.drawString(""+datosComentarioUsuario.getDestino(), xComentario+126, yComentario+134);
			}
			else 
			{
				g.drawImage(cargador.getImagen("msg-usuarioNoGenera.gif"), xComentario, yActual+yComentario, this);
			}
		}
		
		if (comentarioCA1!=null) 
		{
			g.drawImage(comentarioCA1, xComentario, yActual+45, this);
		}
		if (comentarioCA2!=null) 
		{
			g.drawImage(comentarioCA2, xComentario, yActual+104, this);
		}
		
		if (comentarioContadorPeticionesCA1!=null) 
		{
			g.drawImage(comentarioContadorPeticionesCA1, xComentario+9, yActual+45, this);
			if (estaciones[estacionComentada].getPeticionesPendientes(1)>1)
			{
				Celda celdaSal=estaciones[estacionComentada].getCeldaSalida(1);
				if(celdaSal==null || !celdaSal.getOcupada() || celdaSal.getDatos().getOrigen()!=estacionComentada) //Si la celda no ha insertado datos
				{
					g.setFont(new Font("Arial", Font.BOLD, 11));
					g.setColor(Color.black);
			 		g.drawString(""+estaciones[estacionComentada].getPeticionesPendientes(1), xComentario+48, yActual+76);
				}
			}
		}
		if (comentarioContadorPeticionesCA2!=null) 
		{
			g.drawImage(comentarioContadorPeticionesCA2, xComentario+9, yActual+96, this);
			if (estaciones[estacionComentada].getPeticionesPendientes(2)>1)
			{
				Celda celdaSal=estaciones[estacionComentada].getCeldaSalida(2);
				if(celdaSal==null || !celdaSal.getOcupada() || celdaSal.getDatos().getOrigen()!=estacionComentada) //Si la celda no ha insertado datos
				{
					g.setFont(new Font("Arial", Font.BOLD, 11));
					g.setColor(Color.black);
					g.drawString(""+estaciones[estacionComentada].getPeticionesPendientes(2), xComentario+48, yActual+130);
				}
				
			}
		}
		
		if(version==3 || version==4)
			if(comentarioContador!=null) g.drawImage(comentarioContador, xComentario, yComentario, this);

	}
	

	//Mtodo que permite establecer el modo de visibilidad de las estaciones
	public void setVisibilidad(int modoVisibilidad)
	{
		if (modoVisibilidad==VER_TODAS) 
		{
			for(int i=0; i<visibilidad.length;i++) visibilidad[i]=true; 
			this.modoVisibilidad=-1;
			setCursor(FactoriaMultimedia.cursorDefault);

		}
		else if (modoVisibilidad==VER_SELECCIONADAS)
		{
			setCursor(FactoriaMultimedia.cursorMostrarSeleccionadas);
			this.modoVisibilidad=1;
		}
		else if (modoVisibilidad==OCULTAR_TODAS)
		{
			for(int i=0; i<visibilidad.length;i++) visibilidad[i]=false;
			this.modoVisibilidad=-1;
			setCursor(FactoriaMultimedia.cursorDefault);	
		}
		else if (modoVisibilidad==OCULTAR_SELECCIONADAS)
		{
			setCursor(FactoriaMultimedia.cursorOcultarSeleccionadas);
			this.modoVisibilidad=2;
		}
		
		repaint();	
	}
	
	//Mtodo que activa el modo de seleccin de estacin a monitorizar con audio
	public void setModoSeleccionAudio()
	{
		setCursor(FactoriaMultimedia.cursorSeleccionarAudio);
	}
	
	//Mtodo que activa el modo de seleccin de estacin a monitorizar con texto
	public void setModoSeleccionTexto()
	{
		setCursor(FactoriaMultimedia.cursorSeleccionarTexto);
	}
	
	//Mtodo que resetea el icono de estacin monitorizada con audio
	public void resetEstacionMonitorizadaAudio()
	{
		estacionMonitorizadaAudio=-1;
		repaint();
	}
	
	//Mtodo que resetea el icono de estacin monitorizada con texto
	public void resetEstacionMonitorizadaTexto()
	{
		estacionMonitorizadaTexto=-1;
		repaint();
	}
	
	
	//Mtodo establece los eventos ya pasados, para que el mtodo paintComponent los pinte como imgenes estticas
	public void actualizarEventosActivos(int idEstacion)
	{
		eventosActuales[idEstacion-1]=estaciones[idEstacion-1].getEstado().size()-eventosActivos[idEstacion-1];
		eventosActivos[idEstacion-1]=estaciones[idEstacion-1].getEstado().size();

		if (idEstacion-1<celdas1.size())
			 celdas1.elementAt(idEstacion-1).setPeticionRecienInsertada(false);
	
		if (celdas2.size()==numeroEstaciones)
			celdas2.elementAt(idEstacion-1).setPeticionRecienInsertada(false);
		else if (idEstacion-(numeroEstaciones-celdas2.size())-1>=0) 
			celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setPeticionRecienInsertada(false);
	}
	
	//Mtodo que resetea todos los elementos, no dejando ninguna activo
	public void resetearEventosActivos(int idEstacion)
	{
		eventosActivos[idEstacion-1]=0;
		if (idEstacion-1<celdas1.size())
		{
			 celdas1.elementAt(idEstacion-1).setPeticionRecienInsertada(false);
			 celdas1.elementAt(idEstacion-1).setDatosRecienInsertados(false);
		}
		if (celdas2.size()==numeroEstaciones)
		{
			celdas2.elementAt(idEstacion-1).setPeticionRecienInsertada(false);
			celdas2.elementAt(idEstacion-1).setDatosRecienInsertados(false);
		}
		else if (idEstacion-(numeroEstaciones-celdas2.size())-1>=0) 
		{
			celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setPeticionRecienInsertada(false);
			celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setDatosRecienInsertados(false);
		}
	
	}
	
		
	//Mtodo que indica si hay una animacin de celdas en curso o no
	public boolean animacionEnCurso()
	{
		return animacionEnCurso;
	}
	
	//Mtodo que ejecutar el thread que se encargado de animar el movimiento de las celdas
	public void run()
	{
		interfaz.setInicioDeSimulacion();
		animacionEnCurso=true;

		for(int i=0; i<83; i++)
		{
			
			xAnimacion=xAnimacion+2;
			try
			{
				Thread.sleep(50);
			} catch(Exception ex)
			{
				ex.printStackTrace();
			}	
			repaint();
		}
		
		//Establecemos a las celdas la nueva posicion
		for(int i=0; i<celdas1.size(); i++)
		{
			celdas1.elementAt(i).incrementarX(xAnimacion);
			celdas2.elementAt(i).incrementarX(-xAnimacion);
		}
	
		
		//Borramos las celdas que hayan llegado a los terminadores
		if(celdas1.size()==numeroEstaciones+1) celdas1.removeElement(celdas1.lastElement());
		if(celdas2.size()==numeroEstaciones+1) celdas2.removeElementAt(0);
		
		xAnimacion=0;
		animacionEnCurso=false;
		interfaz.setFinDeSimulacion();	
	}
	
	
	//MTODOS DE LA INTERFAZ DQDB_Observer:
	
	//Mtodo que procesa los eventos relativos a la generacin de datos por parte del usuario
	public void notificarDatosGenerados(int idslot, int idevento, int idEstacion, int evento, Datos datos)
	{
		idEventoActual=idevento;
		actualizarEventosActivos(idEstacion);
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos a la aceptacin o rechazo de los datos enviados por el usuario
	public void notificarRecepcionDeDatos(int idslot, int idevento, int idEstacion, int estadoRecepcion, Datos datos)
	{
		idEventoActual=idevento;
		actualizarEventosActivos(idEstacion);
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos a la recepcin de celdas
	public void notificarCeldasRecibidas(int version, int idslot, int idevento, int idEstacion, Celda celdaEntradaBus1, Celda celdaSalidaBus1, Celda celdaEntradaBus2, Celda celdaSalidaBus2, boolean incrementaVacias1, boolean incrementaVacias2)
	{
		idEventoActual=idevento;
		resetearEventosActivos(idEstacion);
		
		if(idEstacion==numeroEstaciones)
		{
			celdas1.insertElementAt(new CeldaGrafica(anchoTerminador-anchoEstacion, margenY+12), 0);
			celdas2.addElement(new CeldaGrafica(anchoTerminador+numeroEstaciones*(anchoEstacion+anchoCanal)+anchoCanal, margenY+171));
			
			if (idevento==-1 || !animarRecepcionDeCeldas)
			{
				//Si simulamos el slot completo (idEvento==-1) o est desactivada la animacin de celdas, no mostramos animacin. Establecemos a las celdas la nueva posicin directamente.
				for(int i=0; i<celdas1.size(); i++)
				{
					celdas1.elementAt(i).incrementarX(166);
					celdas2.elementAt(i).incrementarX(-166);
				}
				
				//Borramos las celdas que hayan llegado a los terminadores
				if(celdas1.size()==numeroEstaciones+1) celdas1.removeElement(celdas1.lastElement());
				if(celdas2.size()==numeroEstaciones+1) celdas2.removeElementAt(0);
			}
			
			//Si solo simulamos un evento y la animacin de celdas est activada, animamos el movimiento
			else 
			{
				Thread thr=new Thread(this);
				thr.start();	
			}
		}
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos al envo de datos desde los Controles de Acceso al usuario
	public void notificarContenidoCeldas(int idslot, int idevento, int idEstacion, int estadoCelda1, int estadoCelda2)
	{
		actualizarEventosActivos(idEstacion);
		repaint();	
	}
	
	//Mtodo que procesa los eventos relativos a la insercin de datos en las celdas
	public void notificarInsercionDeDatosPendientes(int idslot, int idevento, int idEstacion, int estadoDatos1, int estadoDatos2)
	{
		actualizarEventosActivos(idEstacion);
		if (estadoDatos1==Estacion.CA1_INSERTA_DATOS || estadoDatos1==Estacion.CA1_INSERTA_DATOS_V3V4) 
		{
			celdas1.elementAt(idEstacion-1).setDatos(estaciones[idEstacion-1].getCeldaSalida(1).getDatos());
		}
		
		if (estadoDatos2==Estacion.CA2_INSERTA_DATOS || estadoDatos2==Estacion.CA2_INSERTA_DATOS_V3V4)
		{
			if(celdas2.size()==numeroEstaciones) celdas2.elementAt(idEstacion-1).setDatos(estaciones[idEstacion-1].getCeldaSalida(2).getDatos());
			else celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setDatos(estaciones[idEstacion-1].getCeldaSalida(2).getDatos());
		}
		
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos al borrado de celdas
	public void notificarBorradoCelda(int idslot, int idevento, int idEstacion, int estadoCA1, int estadoCA2)
	{
		actualizarEventosActivos(idEstacion);
		
		if (estadoCA1==Estacion.CA1_BORRA_DATOS) 
		{
		
			celdas1.elementAt(idEstacion-1).setOcupada(false);
			celdas1.elementAt(idEstacion-1).setDatos(null);
		}
		if (estadoCA2==Estacion.CA2_BORRA_DATOS) 
		{
			if (celdas2.size()==numeroEstaciones) 
			{
				celdas2.elementAt(idEstacion-1).setOcupada(false);
				celdas2.elementAt(idEstacion-1).setDatos(null);
			}
			else
			{
				celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setOcupada(false);
				celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setDatos(null);
			}
		}
		
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos a la solicitud de envo de peticiones "ro arriba"
	public void notificarSolicitudEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		actualizarEventosActivos(idEstacion);
		repaint();	
	}
	
	//Mtodo que procesa los eventos relativos al envo de la seal RC
	public void notificarEnvioRC(int idslot, int idevento, int idEstacion, int estadoRC1, int estadoRC2, int[] contadorPeticiones, int[] contadorVacias, int[] cuentaAtras)
	{
		actualizarEventosActivos(idEstacion);
		repaint();
	}
	
	//Mtodo que procesa los eventos relativos a la insercin de peticin en las celdas
	public void notificarEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{		
		actualizarEventosActivos(idEstacion);
		
		if (estadoPeticion1==Estacion.CA1_ENVIA_PETICION)
		{
			celdas1.elementAt(idEstacion-1).setPeticion(true);
			celdas1.elementAt(idEstacion-1).setPeticionRecienInsertada(true);
		}
		if (estadoPeticion2==Estacion.CA2_ENVIA_PETICION)
		{
			if (celdas2.size()==numeroEstaciones) 
			{
				celdas2.elementAt(idEstacion-1).setPeticion(true);
				celdas2.elementAt(idEstacion-1).setPeticionRecienInsertada(true);
			}
			else
			{
				celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setPeticion(true);
				celdas2.elementAt(idEstacion-(numeroEstaciones-celdas2.size())-1).setPeticionRecienInsertada(true);
			}
		}
		
	}
	
	//Mtodo que prepara al bloque grfico para iniciar una nueva simulacin, reseteando las variables
	public void notificarNuevaSimulacion(int version, int numeroEstaciones,  double pgUnica, double[] arrayPg, Estacion[] estaciones)
	{
		this.estaciones=estaciones;
		imagenesEstacion[1]=cargador.getImagen("estacionAbiertaVersion"+version+".gif");
		estacionMonitorizadaAudio=-1;
		estacionMonitorizadaTexto=-1;
		this.numeroEstaciones=numeroEstaciones;
		this.version=version;
		visibilidad=new boolean[numeroEstaciones];
		eventosActivos=new int[numeroEstaciones];
		eventosActuales=new int[numeroEstaciones];
		for(int i=0; i<numeroEstaciones; i++)
			visibilidad[i]=true;
		celdas1=new Vector<CeldaGrafica>();
		celdas2=new Vector<CeldaGrafica>();

		redimensionar();
		repaint();
		
	}
	//Mtodo que devuelve la estacin que en cuyo interior queda el punto de coordenadas (x, y). Usado para detectar si el cursor est dento de una estacin o no
	public int getEstacion(int x, int y)
	{
		
		if(x>=margenX+anchoTerminador+anchoCanal)
		{
			x=x-margenX-anchoTerminador-anchoCanal;
			int bloque=anchoEstacion+anchoCanal;
			int indice=x/bloque;
			if(x-indice*bloque>0 && x-indice*bloque<anchoEstacion)
				if(y>margenY && y<margenY+altoEstacion) 
					if(indice<numeroEstaciones) return indice;
		}
		return -1;
		
	}
	
	//Mtodo que devuelve la estacin que se encuentra a la "altura" de la coordenada x especificada. Usado para detectar si el cursor se encuentra dentro de una estacin o en la zona de contadores de las versiones 3 y 4
	public int getEstacion(int x)
	{
		
		if(x>=margenX+anchoTerminador+anchoCanal)
		{
			x=x-margenX-anchoTerminador-anchoCanal;
			int bloque=anchoEstacion+anchoCanal;
			int indice=x/bloque;
			if(x-indice*bloque>0 && x-indice*bloque<anchoEstacion)
					if(indice<numeroEstaciones) return indice;
		}
		return -1;
		
	}
	
	//Mtodo que resetea los mensajes emergentes que explican los elementos de la estacin (seales y dems)
	public void resetearComentarios()
	{
		estacionComentada=-1;
		comentarioContadorPeticionesCA1=null;
		comentarioContadorPeticionesCA2=null;
		comentarioCA1=null;
		comentarioCA2=null;
		datosComentarioUsuario=null;
		comentario=null;
		comentarioContador=null;
	}
	
	//Mtodo que detecta pulsaciones del botn derecho, desplegando el men de configuracin y estadsticas de la estacin sobre la que se ha pulsado (en caso de haberse pulsado sobre una estacin)
	protected void processMouseEvent(MouseEvent e)
	{
		if(modoEspera) return;
    	if(e.isPopupTrigger())
     	{
			//Si se est en modo de seleccin para mostrar u ocultar estaciones, al pulsar el boton derecho se desactivar dicho modo en vez de mostrar el men
			if (modoVisibilidad!=-1)
			{
				modoVisibilidad=-1;
				if(getEstacion(e.getX(), e.getY())==-1) setCursor(FactoriaMultimedia.cursorDefault);
				else setCursor(FactoriaMultimedia.cursorHand);
				return;
			}
			
			//Si se ha pulsado sobre una estacin se despliega el men de estacin
			int indiceEstacion=getEstacion(e.getX(),e.getY());
			if (indiceEstacion!=-1) 
			{
	     		PanelConfiguracionDeEstacion panelConfig=new PanelConfiguracionDeEstacion(this, indiceEstacion+1, estaciones[indiceEstacion]);
				panelConfig.show(e.getComponent(), e.getX(), e.getY()); 
			}
		}
    	else super.processMouseEvent(e);
  	}
	
	
	//Mtodos de la interfaz MouseListener
	public void mousePressed (MouseEvent e)
	{
		if(modoEspera) return;
		
		//Si se est en modo de seleccin de estaciones a mostrar, si se ha pulsado sobre una estacin, pasar a ser "visible"
		if (modoVisibilidad==1) //Mostrar
		{
			int indiceEstacion=getEstacion(e.getX(), e.getY());
			if (indiceEstacion!=-1) 
			{
				visibilidad[indiceEstacion]=true;
				repaint();
			}
		}
		//Si se est en modo de seleccin de estaciones a mostrar, si se ha pulsado sobre una estacin, pasar a ser "no visible"
		else if (modoVisibilidad==2)
		{
			int indiceEstacion=getEstacion(e.getX(), e.getY());
			if (indiceEstacion!=-1) 
			{
				visibilidad[indiceEstacion]=false;
				repaint();
			}
		}
		
		//Si se est en modo de seleccin de estaciones a monitorizar con audio, si se ha pulsado sobre una estacin, pasar a estar monitorizada
		else if (getCursor()==FactoriaMultimedia.cursorSeleccionarAudio) 
		{

			if ((estacionMonitorizadaAudio=getEstacion(e.getX(), e.getY()))!=-1)
			{
				//seleccionamos la nueva estacion
				interfaz.setEstacionMonitorizada(estacionMonitorizadaAudio+1);
				setCursor(FactoriaMultimedia.cursorDefault);
				repaint();
			}
		
		}
		
		//Si se est en modo de seleccin de estaciones a monitorizar con texto, si se ha pulsado sobre una estacin, pasar a estar monitorizada
		else if (getCursor()==FactoriaMultimedia.cursorSeleccionarTexto) 
		{
			
			estacionMonitorizadaTexto=getEstacion(e.getX(), e.getY());
			if (estacionMonitorizadaTexto!=-1)
			{
				interfaz.setEstacionMonitorizadaTexto(estacionMonitorizadaTexto+1);
				setCursor(FactoriaMultimedia.cursorDefault);
				repaint();
			}
		}
		
	} 
	public void mouseReleased (MouseEvent e){}
	public void mouseClicked (MouseEvent e){}
	public void mouseEntered (MouseEvent e){}
	public void mouseExited (MouseEvent e)
	{
		resetearComentarios();
	}
	

	
	//Mtodos de la interfaz MouseMotionListener
	public void mouseDragged (MouseEvent e){}
	
	//Mtodo mouseMoved: comprueba si hay que mostrar algn comentario debido a que se ha movido el cursor del ratn
	public void mouseMoved (MouseEvent e)
	{
		xComentario=-50;
		yComentario=-50;
		if(modoEspera) return;
		int estacion;
		comentarioCeldaOrigen=-1;
		comentarioCeldaDestino=-1;
			
				resetearComentarios();
					
		if(modoVisibilidad==-1 && getCursor()!=FactoriaMultimedia.cursorSeleccionarAudio && getCursor()!=FactoriaMultimedia.cursorSeleccionarTexto) 

	
		
			if ((estacion=getEstacion(e.getX(), e.getY()))!=-1)
			{
				setCursor(FactoriaMultimedia.cursorHand);
				
				int bloque=anchoEstacion+anchoCanal;

				xComentario=margenX+anchoTerminador+anchoCanal+getEstacion(e.getX(), e.getY())*bloque;
				yComentario=58;
				
				int x=e.getX()-xComentario;
				int y=e.getY()-margenY;
				
				
				
				if (!visibilidad[estacion] || !mensajesGraficosActivados) return;
				
				datosComentarioUsuario=null;
				comentarioCA1=null;
				comentarioCA2=null;
				
				if (version==1 || version==2)
				{
				
					//ESTABLECE COMENTARIOS DEL C.A. SUPERIOR (VERSIONES 1 y 2)
					if (estaciones[estacion].getDatosPendientes(1)!=null)
					{
						if (x>2 && x<103 && y>36 && y<52)
						{
							comentarioCA1=cargador.getImagen("msg-CA1DatosPendientes.gif");
						}
					}
					else
					{
						if (x>2 && x<103 && y>36 && y<52)
						{
							Celda celdaSalida=estaciones[estacion].getCeldaSalida(1);
							if(celdaSalida!=null && celdaSalida.getOcupada() && celdaSalida.getDatos().getOrigen()==(estacion+1)) comentarioCA1=cargador.getImagen("msg-CA1InsertaDatos.gif");
							else comentarioCA1=null;
						}
					}
					
					//ESTABLECE COMENTARIOS DEL C.A. INFERIOR (VERSIONES 1 y 2)
					if (estaciones[estacion].getDatosPendientes(2)!=null)
					{
						if (x>2 && x<103 && y>156 && y<170)
						{
							comentarioCA2=cargador.getImagen("msg-CA2DatosPendientes.gif");
						}
					}
					else
					{
						if (x>2 && x<103 && y>156 && y<170)
						{
							Celda celdaSalida=estaciones[estacion].getCeldaSalida(2);
							if(celdaSalida!=null && celdaSalida.getOcupada() && celdaSalida.getDatos().getOrigen()==(estacion+1)) comentarioCA2=cargador.getImagen("msg-CA2InsertaDatos.gif");
							else comentarioCA2=null;
						}
					}
				}
				
				else if (version==3 || version==4)
				{
				
					//ESTABLECE COMENTARIOS DEL C.A. SUPERIOR (VERSIONES 3 y 4)
					if (estaciones[estacion].getDatosPendientes(1)!=null)
					{
						if (x>44 && x<103 && y>36 && y<52)
						{
							comentarioCA1=cargador.getImagen("msg-CA1DatosPendientes.gif");
						}
					}
					else
					{
						if (x>2 && x<103 && y>36 && y<52)
						{
							Celda celdaSalida=estaciones[estacion].getCeldaSalida(1);
							if(celdaSalida!=null && celdaSalida.getOcupada() && celdaSalida.getDatos().getOrigen()==(estacion+1)) comentarioCA1=cargador.getImagen("msg-CA1InsertaDatos.gif");
							else comentarioCA1=null;
						}
					}
					
					if(estaciones[estacion].getPeticionesPendientes(1)>0)
						if (x>2 && x<24 && y>38 && y<50)
						{
							estacionComentada=estacion;
							if(estaciones[estacion].getPeticionesPendientes(1)>1) comentarioContadorPeticionesCA1=cargador.getImagen("msg-CA1PeticionesPendientes.gif");
							else comentarioContadorPeticionesCA1=cargador.getImagen("msg-CA1PeticionPendiente.gif");
						}
					
					//ESTABLECE COMENTARIOS DEL C.A. INFERIOR (VERSIONES 3 y 4)
					if (estaciones[estacion].getDatosPendientes(2)!=null)
					{
						if (x>44 && x<103 && y>156 && y<170)
						{
							comentarioCA2=cargador.getImagen("msg-CA2DatosPendientes.gif");
						}
					}
					else
					{
						if (x>2 && x<103 && y>156 && y<170)
						{
							Celda celdaSalida=estaciones[estacion].getCeldaSalida(2);
							if(celdaSalida!=null && celdaSalida.getOcupada() && celdaSalida.getDatos().getOrigen()==(estacion+1)) comentarioCA2=cargador.getImagen("msg-CA2InsertaDatos.gif");
							else comentarioCA2=null;
						}
					}
					
					if(estaciones[estacion].getPeticionesPendientes(2)>0)
						if (x>2 && x<24 && y>157 && y<169)
						{
							estacionComentada=estacion;
							if(estaciones[estacion].getPeticionesPendientes(2)>1) comentarioContadorPeticionesCA2=cargador.getImagen("msg-CA2PeticionesPendientes.gif");
							else comentarioContadorPeticionesCA2=cargador.getImagen("msg-CA2PeticionPendiente.gif");
						}
				}
				
				

				//ESTABLECE COMENTARIO DE USUARIO
				Shape area;
				area=new Ellipse2D.Float(24,87,53,35);
			    if (area.contains( x,y )) 
				{
					boolean iconoUsuario=false;
					Vector<Integer>listaDeEstado=estaciones[estacion].getEstado();
					for(int j=0; j<listaDeEstado.size()-eventosActuales[estacion]; j++) 
						{
							if(listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA1 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA2 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_NO_GENERA_DATOS);
							else iconoUsuario=true;	
								
						}
						for(int j=listaDeEstado.size()-eventosActuales[estacion]; j<listaDeEstado.size(); j++) 
						{
							if (j>-1)
							{
								if(listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA1 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_GENERA_DATOS_CA2 && listaDeEstado.elementAt(j)!=Estacion.USUARIO_NO_GENERA_DATOS);
								else iconoUsuario=true;	
							}
							
						}
					
					if (iconoUsuario)	
					{	
						datosComentarioUsuario=estaciones[estacion].getDatosGeneradosPorUsuario(); 
						if(datosComentarioUsuario==null) datosComentarioUsuario=new Datos(-1, -1);
						xComentario=margenX+anchoTerminador+anchoCanal+getEstacion(e.getX(), e.getY())*bloque+1;
						repaint();
						return;
					}
				}
		
	
				//ESTABLECE COMENTARIOS DE LAS CELDAS
				if (x>0 && x<105)
				{
					Celda celda;
						
					if (y>12 && y<36) 
					{
						xComentario=margenX+anchoTerminador+anchoCanal+getEstacion(e.getX(), e.getY())*bloque-20;
						
						celda=(estaciones[estacion].getCeldaSalida(1)!=null? estaciones[estacion].getCeldaSalida(1): estaciones[estacion].getCeldaEntrada(1));
						if (celda!=null)
						{
						
			
							if (celda.getOcupada())
							{
								if(celda.getPeticion()) comentario=cargador.getImagen("msg-celdaOcupadaPeticion.gif");
								else comentario=cargador.getImagen("msg-celdaOcupada.gif");
								comentarioCeldaOrigen=celda.getDatos().getOrigen();
								comentarioCeldaDestino=celda.getDatos().getDestino();
								yComentario=-53;
									
							}
							else 
							{
							
								if(celda.getPeticion()) comentario=cargador.getImagen("msg-celdaVaciaPeticion.gif");
								else comentario=cargador.getImagen("msg-celdaVacia.gif");
								yComentario=-35;
							}
							
								
						}
					
						repaint();
						return;
					}
					
					else if (y>172 && y<195)
					{
						xComentario=margenX+anchoTerminador+anchoCanal+getEstacion(e.getX(), e.getY())*bloque-20;
						
						celda=(estaciones[estacion].getCeldaSalida(2)!=null? estaciones[estacion].getCeldaSalida(2): estaciones[estacion].getCeldaEntrada(2));
						if (celda!=null)
						{
							if (celda.getOcupada())
							{
								if(celda.getPeticion()) comentario=cargador.getImagen("msg-celdaOcupadaPeticion.gif");
								else comentario=cargador.getImagen("msg-celdaOcupada.gif");
								
								comentarioCeldaOrigen=celda.getDatos().getOrigen();
								comentarioCeldaDestino=celda.getDatos().getDestino();
								yComentario=107;
							}
							else 
							{
								if(celda.getPeticion()) comentario=cargador.getImagen("msg-celdaVaciaPeticion.gif");
								else comentario=cargador.getImagen("msg-celdaVacia.gif");
								yComentario=125;
							}
						

						}
						repaint();
						return;							
				
					}
					
				
				}
				
				
				//ESTABLECE COMENTARIOS DE LAS SEALES
				comentario=null;
				if (y>58 && y<150)
				{
					Vector<Integer> estadoEstacion;

						if (x>4 && x<8)
						{
							if(estacionContieneEvento(estacion, Estacion.CA1_ENVIA_RC))
								comentario=cargador.getImagen("msg-CA1EnviaRC.gif");
						}	
						
						else if (x>15 && x<23)
						{
							if(estacionContieneEvento(estacion, Estacion.CA2_SOLICITA_ENVIO_DE_PETICION))
								comentario=cargador.getImagen("msg-CA2SolicitaEnvioPeticion.gif");	
						}
						else if (x>75 && x<83)
						{
							if(estacionContieneEvento(estacion, Estacion.CA1_SOLICITA_ENVIO_DE_PETICION))
								comentario=cargador.getImagen("msg-CA1SolicitaEnvioPeticion.gif");
						}	
						else if (x>93 && x<98)
						{
							if(estacionContieneEvento(estacion, Estacion.CA2_ENVIA_RC))
								comentario=cargador.getImagen("msg-CA2EnviaRC.gif");	
						}

					
						else if (y>58 && y<86)
						{
							if (x>31 && x<42) 
							{
								if(estacionContieneEvento(estacion, Estacion.CA1_ENVIA_DATOS_A_USUARIO))
									comentario=cargador.getImagen("msg-CA1EnviaDatos.gif");
							}
							if(x>47 && x<56)
								if(estacionContieneEvento(estacion, Estacion.CA1_BORRA_DATOS))
									comentario=cargador.getImagen("msg-usuarioEnviaBorrarCA1.gif");
							
							if (x>61 && x<69)
							{
								if(estacionContieneEvento(estacion, Estacion.CA1_ACEPTA_DATOS))
									comentario=cargador.getImagen("msg-usuarioEnviaCA1.gif");
								else if(estacionContieneEvento(estacion, Estacion.CA1_RECHAZA_DATOS))	
								comentario=cargador.getImagen("msg-usuarioEnviaCA1Rechazados.gif");
							}
						}
						
						else if (y>123 && y<150)
						{
							if (x>31 && x<42) 
							{
								if(estacionContieneEvento(estacion, Estacion.CA2_ACEPTA_DATOS))
									comentario=cargador.getImagen("msg-usuarioEnviaCA2.gif");
								else if(estacionContieneEvento(estacion, Estacion.CA2_RECHAZA_DATOS))		
									comentario=cargador.getImagen("msg-usuarioEnviaCA2Rechazados.gif");
							}

							if(x>47 && x<56) 
								if(estacionContieneEvento(estacion, Estacion.CA2_BORRA_DATOS))
									comentario=cargador.getImagen("msg-usuarioEnviaBorrarCA2.gif");
							
						if(x>61 && x<69)
								if(estacionContieneEvento(estacion, Estacion.CA2_ENVIA_DATOS_A_USUARIO))
									comentario=cargador.getImagen("msg-CA2EnviaDatos.gif");
						}
						
						else comentario=null;
						
					
				}
			

				repaint();
				
			}
			
			//ESTABLECE COMENTARIOS DE LOS CONTADORES
			else
			{
				if (!mensajesGraficosActivados) return;
			 	setCursor(FactoriaMultimedia.cursorDefault);
				
				if((estacion=getEstacion(e.getX()))!=-1)
				{
					int desfaseContadores=33;
					int y=e.getY();
					int bloque=anchoEstacion+anchoCanal;
					xComentario=margenX+anchoTerminador+anchoCanal+getEstacion(e.getX())*bloque;
					int x=e.getX()-xComentario;

					if (y>margenY-33 && y<margenY-25) 
					{
						if (x>0 && x<13)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY-47;
							comentarioContador=cargador.getImagen("msg-contadorCD1.gif");
						}
					}
					
					else if(y>margenY+236 && y<margenY+243)
					{
						if (x>0 && x<13)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY+122;
							comentarioContador=cargador.getImagen("msg-contadorCD2.gif");
						}
					}
					
					else if(y>margenY-23 && y<margenY-15)
					{
						if (x>0 && x<39)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY-42;
							comentarioContador=cargador.getImagen("msg-contadorVaciasCA1.gif");
						}
					}
					
					else if(y>margenY+226 && y<margenY+233)
					{
						if (x>0 && x<39)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY+194;
							comentarioContador=cargador.getImagen("msg-contadorVaciasCA2.gif");
						}
					}
					
					else if(y>margenY-13 && y<margenY-5)
					{
						if (x>0 && x<62)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY-45;
							comentarioContador=cargador.getImagen("msg-contadorPeticiones.gif");
						}
					}
					
					else if(y>margenY+216 && y<margenY+223)
					{
						if (x>0 && x<57)
						{
							xComentario=margenX+anchoTerminador+(estacion+1)*anchoCanal+estacion*anchoEstacion;
							yComentario=margenY+184;
							comentarioContador=cargador.getImagen("msg-contadorPeticiones.gif");
						}
					}
				}
				
				repaint();
			}
	}
	
	//Mtodo que determina si en una estacin ha ocurrido un evento determinado
	public boolean estacionContieneEvento(int idEstacion, int evento)
	{
		Vector<Integer> estado=estaciones[idEstacion].getEstado();
		for(int i=0; i<estado.size(); i++)
		{
			if(estado.elementAt(i)==evento) return true;
		}
		return false;
	}
	
	
	//Mtodo que activa/dewaciva el modo espera (cambia el cursor a un reloj mientras se est realizando la simulacin)
	public void setModoEspera(boolean modo)
	{
			modoEspera=modo;
			Cursor cursor=(modo? FactoriaMultimedia.cursorWait: FactoriaMultimedia.cursorDefault);
			setCursor(cursor);
	}
	
	//Mtodo que devuelve la visibilidad de una estacin concreta
	public boolean getVisibilidad(int idEstacion)
	{
		return visibilidad[idEstacion];
	}
	
	//Mtodo que devuelve el array que indica la visibilidad de cada estacin
	public boolean[] getArrayVisibilidad()
	{
		return visibilidad;
	}
	
	//mtodo que indica al bloque grfico si debe mostrar mensajes explicativos al situar el cursor sobre los elementos de la estacin
	public void setMensajesGraficos(boolean b)
	{
		mensajesGraficosActivados=b;
	}
}