import java.util.Vector;

//CLASE QUE IMPLEMENTA EL BLOQUE DE SIMULACIN, ENCARGADO DE SIMULAR EL COMPORTAMIENTO DE LA RED Y NOTIFICAR A LOS OBSERVADORES DE LOS EVENTOS QUE VAYAN OCURRIENDO
public class BloqueSimulacion
{
	private static int EVENTO=1, SLOT=2;
	private Vector<DQDB_Observer> observadores; //Lista de observadores
	private int numeroEstaciones, version;
	
	private Estacion[] estaciones;

	private int evento;
	private int slotActual;
	private boolean flagSlotCompleto;
	private int simular;
	
	//CONSTRUCTOR
	public BloqueSimulacion(int version, int numeroEstaciones, double pgUnica, double[] arrayPg)
	{
		slotActual=1;
		this.numeroEstaciones=numeroEstaciones;
		this.version=version;
		observadores=new Vector<DQDB_Observer>();

		estaciones=new Estacion[numeroEstaciones];
		for(int i=0; i<estaciones.length; i++) 
		{
			double pg=(arrayPg!=null? arrayPg[i]: pgUnica);
			if(version==1) estaciones[i]=new EstacionVersion1(i+1, pg, numeroEstaciones, version);
			else if(version==2) estaciones[i]=new EstacionVersion2(i+1, pg, numeroEstaciones, version);
			else if(version==3) estaciones[i]=new EstacionVersion3(i+1, pg, numeroEstaciones, version);
			else if(version==4) estaciones[i]=new EstacionVersion4(i+1, pg, numeroEstaciones, version);
		}
	
	}
	
	//Mtodo que inserta un observador en la lista de observadores
	public void addObserver(DQDB_Observer observador)
	{
		observadores.add(observador);
	}
	
	//Mtodo que crea una nueva red y avisa a los observadores de que reseteen cualquier informacin que contengan sobre la simulacin anterior
	public void crearNuevaSimulacion(int version, int numeroEstaciones, double pgUnica, double[] arrayPg)
	{
		slotActual=1;
		evento=0;
		this.numeroEstaciones=numeroEstaciones;
		this.version=version;
		estaciones=new Estacion[numeroEstaciones];

		for(int i=0; i<estaciones.length; i++) 
		{
			double pg=(arrayPg!=null? arrayPg[i]: pgUnica);
			if(version==1) estaciones[i]=new EstacionVersion1(i+1, pg, numeroEstaciones, version);
			else if(version==2) estaciones[i]=new EstacionVersion2(i+1, pg, numeroEstaciones, version);
			else if(version==3) estaciones[i]=new EstacionVersion3(i+1, pg, numeroEstaciones, version);
			else if(version==4) estaciones[i]=new EstacionVersion4(i+1, pg, numeroEstaciones, version);
		}
		
		notificarNuevaSimulacion(version, numeroEstaciones, pgUnica, arrayPg, estaciones);
	}
	
	
	//Mtodo que devuelve la lista de estaciones de la red simulada
	public Estacion[] getEstaciones()
	{
		return estaciones;
	}
	
	//Mtodo que permite cambiar la PG de una estacin
	public void cambiarPg(int numeroEstacion, double nuevaPg)
	{
		estaciones[numeroEstacion-1].setPg(nuevaPg);
	}


	//Mtodo que permite simular un cierto nmero de slots
	public void simularSlot(int numeroSlots)
	{
		flagSlotCompleto=true;
			
		//Si hay un slot a medio simular lo completamos )simulamos los eventos restantes)
		if (evento!=0)
		{
			while(evento!=0) simularEvento(true);
			numeroSlots--;
			if(numeroSlots==0) return; //Si slo haba que simular un slot terminamos.
		}
		
		
		//Si el slot no est a medio lo simulamos desde el principio o lo estaba pero haba que simular ms de uno, simulamos los slots requeridos
		
		//VERSION 1
		if (version==1)
		{
			for(int i=0; i<numeroSlots; i++)
			{
				recibirCeldas();
				comprobarSiUsuariosGeneranDatos();
				comprobarSiDatosSonAceptados();
				comprobarSiCeldasEstanOcupadas();
				comprobarSiHayDatosPendientes();
				slotActual++;	
			}
				
		}
		
		//VERSION 2
		else if (version==2)
		{
				for(int i=0; i<numeroSlots; i++)
				{
					recibirCeldas();
					comprobarSiUsuariosGeneranDatos();
					comprobarSiDatosSonAceptados();
					comprobarSiCeldasEstanOcupadas();
					comprobarSiHayQueBorrarCelda();
					comprobarSiHayDatosPendientes();
					slotActual++;
				}
				
		}
		
		//VERSIN 3
		else if (version==3)	
		{
			for(int i=0; i<numeroSlots; i++)
				{
					recibirCeldas();
					comprobarSiCeldasEstanOcupadas();
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					if (algunUsuarioGenera)
					{
						comprobarSiDatosSonAceptados();
						comprobarSolicitudEnvioDePeticion();
						comprobarEnvioRC();
					
					}
					comprobarEnvioDePeticion();
					comprobarSiHayDatosPendientes();
					slotActual++;
				}
		}
		
		//VERSIN 4
		else if (version==4)	
		{
			for(int i=0; i<numeroSlots; i++)
				{			
					recibirCeldas();
					
					comprobarSiCeldasEstanOcupadas();
					comprobarSiHayQueBorrarCelda();
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					if (algunUsuarioGenera)
					{
						comprobarSiDatosSonAceptados();
						comprobarSolicitudEnvioDePeticion();
						comprobarEnvioRC();
					
					}
					comprobarEnvioDePeticion();
					comprobarSiHayDatosPendientes();
					slotActual++;
				}
		}
	}
	
	
	//Mtodo que permite simular el siguiente evento
	public void simularEvento (boolean simularSemiSlot)
	{
		flagSlotCompleto=(simularSemiSlot? true: false); //flagSlotCompleto indicar a los observadores que se ha simulado un slot completo (si entra aqu significa que se est simulando un slot cuando ya estaba a medio simular por eventos)
		
		//VERSIN 1
		if (version==1)
		{
			switch (evento)	
			{	
	
				case 0: 
					recibirCeldas();
					evento=1;
					break;
				case 1: 
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					evento=(algunUsuarioGenera? 2: 3);
					break;

				case 2: 
					comprobarSiDatosSonAceptados();
					evento=3;
					break;

				case 3: 
					comprobarSiCeldasEstanOcupadas();
					evento=4;
					break;

				case 4: 
					comprobarSiHayDatosPendientes();
					evento=0;
					break;

			}
		
				
		}
		
		//VERSIN 2
		else if (version==2)
		{
	
			switch (evento)	
			{	
				case 0: 
					recibirCeldas();
					evento=1;
					break;
				case 1: 
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					evento=(algunUsuarioGenera? 2: 3);
					break;

				case 2: 
					comprobarSiDatosSonAceptados();
					evento=3;
					break;
	
				case 3: 
					comprobarSiCeldasEstanOcupadas();
					evento=4;
					break;

				case 4: 
					comprobarSiHayQueBorrarCelda();
					evento=5;
					break;

				case 5: 
					comprobarSiHayDatosPendientes();
					evento=0;
					break;
			} 
		
				
		} 
		
		//VERSIN 3
		else if (version==3)	
		{
			switch (evento)	
			{	
				case 0: 
					recibirCeldas();
					evento=1;	
					break;
						
				case 1: 
					comprobarSiCeldasEstanOcupadas();
					evento=2;
					break;
						
				case 2: 
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					evento=(algunUsuarioGenera? 3: 6);
					break;
					
				case 3: 
					comprobarSiDatosSonAceptados();
					evento=4;					
					break;
		
				case 4:
					comprobarSolicitudEnvioDePeticion();
					evento=5;
					break;

				case 5:
					comprobarEnvioRC();
					evento=6;
					break;	

				case 6:
					comprobarEnvioDePeticion();
					evento=7;
					break;
					
				case 7: 
					comprobarSiHayDatosPendientes();
					evento=0;
					break;
				} 			
		}
		
		//VERSIN 4
		else if (version==4)	
		{
	
			switch (evento)	
			{	
				case 0: 
					recibirCeldas();
					evento=1;	
					break;
											
				case 1: 
					comprobarSiCeldasEstanOcupadas();
					evento=2;
					break;
					
				case 2: 
					comprobarSiHayQueBorrarCelda();
					evento=3;
					break;
						
				case 3: 
					boolean algunUsuarioGenera=comprobarSiUsuariosGeneranDatos();
					evento=(algunUsuarioGenera? 4: 7);
					break;

				case 4: 
					comprobarSiDatosSonAceptados();
					evento=5;		
					break;

				case 5:
					comprobarSolicitudEnvioDePeticion();
					evento=6;
					break;
	
				case 6:
					comprobarEnvioRC();
					evento=7;
					break;

				case 7:
					comprobarEnvioDePeticion();
					evento=8;
					break;

				case 8: 
					comprobarSiHayDatosPendientes();
					evento=0;
					break;
					
			} 
		}
		
		if(evento==0) slotActual++;
		
	}
	
		
	//MTODOS DE SIMULACION:
	
	//Mtodo que comprueba si lso usuarios generan datos
	public boolean comprobarSiUsuariosGeneranDatos()
	{	
		boolean algunUsuarioGenera=false;
		for(int i=0; i<estaciones.length; i++)
		{
			int estado=estaciones[i].comprobarSiUsuarioGeneraDatos();
			Datos datos= estaciones[i].getDatosGeneradosPorUsuario();
			if(datos!=null) algunUsuarioGenera=true;//Si no genera ninguno se omite el evento de comprobar si los CA aceptan o rechazan datos
			notificarDatosGenerados((i+1), estado,datos);
		}
		
		return algunUsuarioGenera;
	}
	
	//Mtodo que comprueba si los CA aceptan o rechazan datos
	public void comprobarSiDatosSonAceptados()
	{	
		for(int i=0; i<estaciones.length; i++)
		{
			Datos datos=estaciones[i].getDatosGeneradosPorUsuario();
				notificarRecepcionDeDatos((i+1), estaciones[i].comprobarSiDatosSonAceptados(), datos);
		}
	}
	
	//Mtodo que provoca el envo de celdas an ambos buses
	public void recibirCeldas()
	{	

		Celda celdaEntrada1, celdaEntrada2, celdaSalida1, celdaSalida2;
		
		//Para cada estacin
		for(int i=0; i<estaciones.length; i++)
		{
		
			celdaSalida1=estaciones[i].getCeldaSalida(1);
			celdaSalida2=estaciones[i].getCeldaSalida(2);

			celdaEntrada1=(i==0? new Celda(): estaciones[i-1].getCeldaSalida(1));
			celdaEntrada2=(i==numeroEstaciones-1? new Celda(): estaciones[i+1].getCeldaSalida(2));
		
			boolean incrementaVacias1=false, incrementaVacias2=false;
			if (version==3 || version==4)
			{
				//Si se trata de la versin 3 o 4:
				
				//Incrementa el contador de peticiones en caso de recibirse una celda marcada con peticin
				if(celdaEntrada1!=null && celdaEntrada1.getPeticion()) estaciones[i].incrementarContadorPeticiones(1);
				if(celdaEntrada2!=null && celdaEntrada2.getPeticion()) estaciones[i].incrementarContadorPeticiones(2);
				
				//Si la estacin ha enviado una celda vaca la contabiliza (el contador afectado depender de si CD est activo o no)
				if(celdaSalida1!=null && !celdaSalida1.getOcupada()) incrementaVacias1=estaciones[i].contabilizarCeldaVacia(1);
				if(celdaSalida2!=null && !celdaSalida2.getOcupada()) incrementaVacias2=estaciones[i].contabilizarCeldaVacia(2);
			}
			
			
			if(i!=0) estaciones[i-1].setCeldaSalida(1, null);
			
			
			estaciones[i].setCeldaSalida(2, null);

			
			estaciones[i].setCeldaEntrada(1, celdaEntrada1);
			estaciones[i].setCeldaEntrada(2, celdaEntrada2);
			
			
			notificarCeldasRecibidas((i+1), celdaEntrada1, celdaSalida1, celdaEntrada2, celdaSalida2, incrementaVacias1, incrementaVacias2);
			
			
		}
		
		
	}
	
	//Mtodo que comprueba si la sceldas estn vacas u ocupadas
	public void comprobarSiCeldasEstanOcupadas()
	{
		for(int i=0; i<estaciones.length; i++)
			notificarContenidoCeldas((i+1), estaciones[i].comprobarSiCeldaContieneDatos(1), estaciones[i].comprobarSiCeldaContieneDatos(2));
	}

	//Mtodo que comprueba si las celdas son borradas
	public void comprobarSiHayQueBorrarCelda()
	{
		int estadoCelda1, estadoCelda2;
		
		for(int i=0; i<estaciones.length; i++)
		{
			estadoCelda1=estaciones[i].borrarCelda(1);
			estadoCelda2=estaciones[i].borrarCelda(2);
			notificarBorradoCelda(i+1, estadoCelda1, estadoCelda2);
		}
	}
	
	//Mtodo que comprueba si los CA insertan datos
	public void comprobarSiHayDatosPendientes()
	{
		for(int i=0; i<estaciones.length; i++)
			notificarInsercionDeDatosPendientes((i+1), estaciones[i].comprobarDatosPendientesCA(1), estaciones[i].comprobarDatosPendientesCA(2));
	}
	
	//Mtodo que comprueba si lso CA solicitan envo de una peticin "ro arriba"
	public void comprobarSolicitudEnvioDePeticion()
	{
		for(int i=0; i<estaciones.length; i++)
			notificarSolicitudEnvioDePeticion((i+1), estaciones[i].comprobarSolicitudEnvioDePeticion(1), estaciones[i].comprobarSolicitudEnvioDePeticion(2));
	}
	
	//Mtodo que comprueba si los CA envan la seal RC
	public void comprobarEnvioRC()
	{
		for(int i=0; i<estaciones.length; i++)
			notificarEnvioRC((i+1), estaciones[i].comprobarEnvioRC(1), estaciones[i].comprobarEnvioRC(2), estaciones[i].getContadorVacias(), estaciones[i].getContadorPeticiones(), estaciones[i].getCuentaAtras());
	}
	
	//Mtodo que comprueba si los CA marcan una celda com peticin
	public void comprobarEnvioDePeticion()
	{
		for(int i=0; i<estaciones.length; i++)
			notificarEnvioDePeticion((i+1), estaciones[i].comprobarEnvioDePeticion(1), estaciones[i].comprobarEnvioDePeticion(2));
	}
	
	//MTODOS QUE NOTIFICAN A LOS OBSERVADORES DEL RESULTADO DE LOS MTODOS DE SIMULACIN QUE ACABAMOS DE VER:
	public void notificarEnvioDePeticion(int idEstacion, int estadoPeticion1,  int estadoPeticion2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarEnvioDePeticion(slotActual, evento, idEstacion, estadoPeticion1, estadoPeticion2);
	}
	
	
	public void notificarDatosGenerados(int idEstacion, int estadoRecepcion, Datos datos)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarDatosGenerados(slotActual, evento, idEstacion, estadoRecepcion, datos);
	}
	
	public void notificarRecepcionDeDatos(int idEstacion, int eventoRecepcion, Datos datos)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarRecepcionDeDatos(slotActual, evento, idEstacion, eventoRecepcion, datos);
	}
	
	public void notificarCeldasRecibidas(int idEstacion, Celda celdaRecibidaBus1, Celda celdaEnviadaBus1, Celda celdaRecibidaBus2, Celda celdaEnviadaBus2, boolean incrementaVacias1, boolean incrementaVacias2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarCeldasRecibidas(version, slotActual, evento, idEstacion, celdaRecibidaBus1, celdaEnviadaBus1, celdaRecibidaBus2, celdaEnviadaBus2, incrementaVacias1, incrementaVacias2);
	}
	
	public void notificarContenidoCeldas(int idEstacion, int estadoCelda1, int estadoCelda2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarContenidoCeldas(slotActual, evento, idEstacion, estadoCelda1, estadoCelda2);
	}
	
	public void notificarInsercionDeDatosPendientes(int idEstacion, int estadoDatos1, int estadoDatos2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarInsercionDeDatosPendientes(slotActual, evento, idEstacion, estadoDatos1,estadoDatos2);
	}
	
	public void notificarBorradoCelda(int idEstacion, int estadoCelda1, int estadoCelda2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarBorradoCelda(slotActual, evento, idEstacion, estadoCelda1,estadoCelda2);
	}
	
	public void notificarSolicitudEnvioDePeticion(int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarSolicitudEnvioDePeticion(slotActual, evento, idEstacion, estadoPeticion1,estadoPeticion2);
		
	}
	
	public void notificarEnvioRC(int idEstacion, int estadoRC1, int estadoRC2, int[] contadorVacias, int[] contadorPeticiones, int[] cuentaAtras)
	{
		int evento=(flagSlotCompleto?-1: this.evento);
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarEnvioRC(slotActual, evento, idEstacion, estadoRC1,estadoRC2, contadorPeticiones, contadorVacias, cuentaAtras);
		
	}
	
	public void notificarNuevaSimulacion(int version, int numeroEstaciones,  double pgUnica, double[] arrayPg, Estacion[] estaciones)
	{
		for(int i=0; i<observadores.size(); i++)
			observadores.elementAt(i).notificarNuevaSimulacion(version, numeroEstaciones, pgUnica, arrayPg, estaciones);
	}
	
	
	
	
	
}


























				

	