import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.font.*;

//CLASE QUE IMPLEMENTA LAS GRFICAS SIMPLES
public class Grafica extends JPanel
{
	private Image infinito;
	private int marcadorDeNavegacion;
	private int infinitos;
	protected boolean esZoom;
	protected BloqueEstadisticas listener;
	protected int xCursor, yCursor;
	private Grafica graficaZoom;
	protected int idEstacion;
	protected int margenY=20;
	protected int origenX=65,origenY=margenY+260, TamEjeX=400,TamEjeY=220;
	private Vector<Punto> puntos;
	protected double xmax=0,ymax=0;
	protected Image background, labelX, labelY, titulo;
	protected int separadores=10;

	//CONSTRUCTOR (SIN ZOOM)
	public Grafica(Image background, Image labelX, Image labelY, Image titulo, int idEstacion, BloqueEstadisticas listener)
	{
		infinito=CargadorDeImagenes.getCargador().getImagen("infinito.gif");
		xCursor=-1;
		yCursor=-1;
		this.listener=listener;
		esZoom=false;
		addMouseListener(listener);
		addMouseMotionListener(listener);
		this.background=background;
		this.titulo=titulo;
		this.labelX=labelX;
		this.labelY=labelY;
		
		this.idEstacion=idEstacion;
		
		Dimension dim=new Dimension(background.getWidth(this),background.getHeight(this)+margenY);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);
		
		puntos=new Vector<Punto>();
		insertarPunto(0,0);
	}

	//CONSTRUCTOR 2 (CON ZOOM)
	public Grafica(Image background, Image labelX, Image labelY, Image titulo, int idEstacion, Vector<Punto> puntos, int separadores, double xmax, double ymax,  int marcadorDeNavegacion, BloqueEstadisticas listener)
	{
		infinito=CargadorDeImagenes.getCargador().getImagen("infinito.gif");
		xCursor=-1;
		yCursor=-1;
		esZoom=true;
		addMouseListener(listener);
		addMouseMotionListener(listener);
		this.separadores=separadores;
		this.puntos=puntos;
		margenY=5;
		origenX=65;
		origenY=margenY+585;
		TamEjeX=650;
		TamEjeY=550;
		this.xmax=xmax;
		this.ymax=ymax;
		this.background=background;
		this.titulo=titulo;
		this.labelX=labelX;
		this.labelY=labelY;
		
		this.idEstacion=idEstacion;
		this.marcadorDeNavegacion=marcadorDeNavegacion;
		
		Dimension dim=new Dimension(background.getWidth(this),background.getHeight(this)+margenY);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);

	}
	
	//Mtodo que inserta un punto en la grfica
	public void insertarPunto(double x, double y)
	{
		if(x==-1 || y==-1) infinitos++;
		marcadorDeNavegacion++;
		if(x>xmax)xmax=x;
		if(y>ymax)ymax=y;

		puntos.addElement(new Punto(x, y));
		
		if (graficaZoom!=null)
		{
			graficaZoom.updateMaximos(xmax, ymax);
			graficaZoom.incrementarMarcadorDeNavegacion();
		}
		repaint();
	}

	//Mtodo que devuelve la lista de puntos de la grfica
	public Vector<Punto> getPuntos()
	{
		return puntos;
	}


	//Mtodo que indica si la grfica es una grfica zoom o no
	public boolean esZoom()
	{
		return esZoom;
	}

	//Mtodo que provoca que la grfica detecte y muestre el punto sobre el que est el cursor
	public void setCoordenadasCursor(int x, int y)
	{
		xCursor=x;
		yCursor=y;
		repaint();
	}

	//Establece el marcador de navegacin, es decir, el ndice del ltimo punto de la lista a mostar en la grfica.
	public void setMarcadorDeNavegacion(int valor)
	{
		marcadorDeNavegacion=valor;
		actualizarMaximos();
		repaint();
	}


	//Mtodo que incrementa en una unidad el marcador de navegacin
	public boolean incrementarMarcadorDeNavegacion()
	{
		if (marcadorDeNavegacion>=puntos.size()) return false;

		marcadorDeNavegacion++;
		actualizarMaximos();
		repaint();
		return true;
	}
	
	//Mtodo que decrementa en una unidad el marcador de navegacin
	public boolean decrementarMarcadorDeNavegacion()
	{
		if(marcadorDeNavegacion==0) return false;
		marcadorDeNavegacion--;
		actualizarMaximos();
		repaint();
		return false;
	}


	//Mtodo que calcula las coordenadas x mxima e y mxima en el rango comprendido entre "0 y marcadorDeNavegacin", necesarias para poder establecer la escala de los ejes y dicujar los puntos
	public void actualizarMaximos()
	{
		double nuevaXmax=0;
		double nuevaYmax=0;
		Punto punto;
		int imax=Math.min(marcadorDeNavegacion, puntos.size());
		for(int i=0; i<imax; i++)
		{
			punto=puntos.elementAt(i);
			if(nuevaXmax<punto.getX()) nuevaXmax=punto.getX();
			if(nuevaYmax<punto.getY()) nuevaYmax=punto.getY();
		
		}
		
		xmax=nuevaXmax;
		ymax=nuevaYmax;
	}

	//Mtodo que actualiza las variables x mxima e y mxima a los valores especificados
	public void updateMaximos(double xmax, double ymax)
	{
		this.xmax=xmax;
		this.ymax=ymax;
		repaint();
	}

	//Mtodo que elimina los puntos de la grfica
	public void reset()
	{
		puntos=new Vector<Punto>();
		puntos.addElement(new Punto(0,0));
		xmax=0;
		ymax=0;	
		repaint();
	}

	//Mtodo que genera una rplica de esta grfica pero de mayor tamao
	public Grafica zoom()
	{
		return graficaZoom=new Grafica(CargadorDeImagenes.getCargador().getImagen("graficaZoom.gif"), labelX, labelY, titulo, idEstacion, puntos, 20, xmax, ymax, marcadorDeNavegacion, listener);
	}

	
	public void eliminarZoom()
	{
		graficaZoom=null;
	}



	public void paintComponent( Graphics g )
	{
		super.paintComponent(g);

		Graphics2D graphics2d=(Graphics2D) g;
		graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		graphics2d.setFont(new Font("Arial", Font.BOLD, 20));
		
		g.drawImage(background, 0, margenY, this);
		int largoTexto=0;
		if (idEstacion!=-1)
		{
			Font fuente=graphics2d.getFont();
			FontRenderContext frc = graphics2d.getFontRenderContext();
			LineMetrics linemetrics = fuente.getLineMetrics(""+idEstacion, frc);
			largoTexto= (int)fuente.getStringBounds(""+idEstacion, frc).getWidth();
			graphics2d.drawString(""+idEstacion, (background.getWidth(this)-titulo.getWidth(this)-largoTexto)/2+titulo.getWidth(this), margenY+23);

		}
		
		//Dibuja el ttulo de la grfica y los nombres de los ejes
		g.drawImage(titulo, (background.getWidth(this)-titulo.getWidth(this)-largoTexto)/2, margenY+7, this);
		g.drawImage(labelX, (background.getWidth(this)-labelX.getWidth(this))/2, margenY+background.getHeight(this)-20, this);
		g.drawImage(labelY,10, margenY+(background.getHeight(this)-labelY.getHeight(this))/2, this);
		
		//Dibuja el rea de la grfica que contendr los puntos
		g.setColor(Color.white);
		g.fillRect(origenX, origenY-TamEjeY,TamEjeX,TamEjeY);
		g.setColor(Color.lightGray);
		g.drawLine(origenX+TamEjeX, origenY-TamEjeY, origenX+TamEjeX,origenY);
		g.drawLine(origenX, origenY-TamEjeY, origenX+TamEjeX,origenY-TamEjeY);
		
		g.setColor(Color.black);

		
		
		g.setFont(new Font("Verdana", Font.PLAIN, 9));

		//pinta los valores de los separadores del ejes
		if (ymax>0)	
		{		
			for(int i=1;i<=separadores;i++)
			{
				if (ymax>=10)
					g.drawString( ""+ (Math.round(i*ymax*100/separadores)/100.0),  origenX-35,  origenY-(int)Math.round(i*TamEjeY/separadores) +5);
		
				else g.drawString( ""+ (Math.round(i*ymax*1000/separadores)/1000.0),  origenX-35,  origenY-(int)Math.round(i*TamEjeY/separadores) +5);	

			}	
		}
		
		if ((xmax>0))	
		{		
			for(int i=1;i<=separadores;i++)
			{
				if (xmax>=10){g.drawString( ""+ (Math.round(i*xmax*100/separadores)/100.0),  origenX+(int)Math.round(1*i*TamEjeX/separadores)-8, origenY+13 );}
				else g.drawString( ""+ (Math.round(i*xmax*1000/separadores)/1000.0),  origenX+(int)Math.round(1*i*TamEjeX/separadores)-8, origenY+13 );	
			}	
		}		
		
		
		//pinta proyecciones sobre los ejes
		g.setColor(new Color(230,230,230));	
		int imax=Math.min(marcadorDeNavegacion, puntos.size());
		if(puntos.size()>1)for(int i=0;i<imax;i++)
		{
			g.drawLine(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax), origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax), origenX+1,origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax));
			g.drawLine(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax), origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax), origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax),origenY-1);
		}

		//pinta lineas de que unen los puntos
		g.setColor(new Color(0,0,0));
		if(puntos.size()>1)
			for(int i=1;i<imax;i++)
			{
				if(puntos.elementAt(i).getY()>-1 && puntos.elementAt(i-1).getY()!=-1) 
					g.drawLine(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax), origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax) , origenX+(int)Math.round(TamEjeX*puntos.elementAt(i-1).getX()/xmax), origenY-(int)Math.round(TamEjeY*puntos.elementAt(i-1).getY()/ymax) );
			}
				
		
		//pinta puntos de la grafica	
		g.setColor(new Color(30,30,240));
		int inf=infinitos;		
		if(puntos.size()>1) for(int i=0;i<imax;i++)
		{
				g.setColor(Color.blue);
			if(puntos.elementAt(i).getY()>-1)
	    	g.fillOval(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax)-2, origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax)-2,5, 5);  	   
		
			else// if(puntos.elementAt(i-1).getY()>-1)
			{
				int xPunto=origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax);
				g.setColor(Color.red);
				if(inf==1) g.drawImage(infinito,  xPunto-infinito.getWidth(this)/2, origenY-TamEjeY-infinito.getHeight(this)-5, this);
				else inf--;
				g.fillRect(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax)-2,origenY-TamEjeY-4,5, 5);
			}	   
		}
		
		
	
		//Si el cusor est sobre la grfica comprobamos si cae dentro del radio de accin de algn punto
		int indicePuntoMasCercano=-1;
		int menorDistancia=10;
		if (xCursor!=-1)
		{
			if(puntos.size()>1)
				for(int i=1;i<imax;i++)
				{
					int x=origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax); 
			   		int y=origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax);
			   	
					
					if (puntos.elementAt(i).getY()!=-1)
					{
						if ((Math.abs(x-xCursor)+Math.abs(y-yCursor)) < menorDistancia )
						{
							indicePuntoMasCercano=i;
							menorDistancia=Math.abs(x-xCursor)+Math.abs(y-yCursor);
						}
					}
					else
					{
						if ((Math.abs(x-xCursor)+Math.abs(origenY-TamEjeY-yCursor)) < menorDistancia )
						{
							indicePuntoMasCercano=i;
							menorDistancia=Math.abs(x-xCursor)+Math.abs(y-yCursor);
						}
					}
				}
						
			Punto punto;
			
			//Si ese ha encontrado un punto en cuyo radio de accin se encuentra el puntero mostramos la informacin del punto
			if (indicePuntoMasCercano!=-1 && (punto=puntos.elementAt(indicePuntoMasCercano)).getY()!=-1)
			{
				
				g.setColor(Color.red);
				g.drawOval(origenX+(int)Math.round(TamEjeX*puntos.elementAt(indicePuntoMasCercano).getX()/xmax)-4, 
					   origenY-(int)Math.round(TamEjeY*puntos.elementAt(indicePuntoMasCercano).getY()/ymax)-4,8,8);  
					
					  
				//Escribimos la informacin del punto suavizando el texto
				String textoPuntoSeleccionado="["+punto.getX()+", "+(punto.getY()!=-1? punto.getY(): "infinito")+"]";
				graphics2d.setFont(new Font("Verdana", Font.PLAIN, 9));
				Font fuente=graphics2d.getFont();
				FontRenderContext frc = graphics2d.getFontRenderContext();
				LineMetrics linemetrics = fuente.getLineMetrics(textoPuntoSeleccionado, frc);
				int largo = (int)fuente.getStringBounds(textoPuntoSeleccionado, frc).getWidth();
				int alto=(int)fuente.getStringBounds(textoPuntoSeleccionado, frc).getHeight();
					
					
				graphics2d.setColor(Color.white);
			  	g.fillRect(origenX+(int)Math.round(TamEjeX*punto.getX()/xmax), origenY-(int)Math.round(TamEjeY*punto.getY()/ymax)+24, largo+2, alto+2);
				graphics2d.setColor(Color.black);
				g.drawRect(origenX+(int)Math.round(TamEjeX*punto.getX()/xmax)-1, origenY-(int)Math.round(TamEjeY*punto.getY()/ymax)+24-1, largo+2+1, alto+2+1);
				graphics2d.setColor(new Color(255,0,255));
				g.drawString(textoPuntoSeleccionado, origenX+(int)Math.round(TamEjeX*punto.getX()/xmax)+1 , origenY-(int)Math.round(TamEjeY*punto.getY()/ymax)+24+alto-1);	   
			}
		}
	
		//Pinta los ejes de la gfica
		g.setColor(Color.black);
		g.drawLine(origenX, origenY-TamEjeY, origenX,origenY);
		g.drawLine(origenX, origenY, origenX+TamEjeX,origenY);
		
		//Pinta los separadores de ejes
		int ySeparador, xSeparador;
		for(int i=1;i<=separadores;i++)
		{
			//Eje vertical
			ySeparador=origenY-(int)Math.round(i*TamEjeY/separadores);
			g.drawLine(origenX-3, ySeparador , origenX+3, ySeparador);	
			
			//Eje horizontal
			xSeparador=origenX+(int) Math.round(i*TamEjeX/separadores);
			g.drawLine(xSeparador, origenY-3 , xSeparador, origenY+3 );

		}

	}


}