import  sun.audio.*;
import java.net.URL;
import java.applet.*;

//CLASE QUE IMPLEMENTA EL OBSERVADOR DEL BLOQUE DE SIMULACIN QUE SE ENCARGA DE REPRODUCIR SONIDOS RELATIVOS A LOS EVENTOS QUE OCURREN EN UNA DETERMINADA ESTACIN
public class BloqueAudio implements DQDB_Observer
{
	private FactoriaMultimedia factoria=FactoriaMultimedia.getFactoria();
	private int idEstacionMonitorizada;
	private AudioStream as;
	private AudioClip ac;
	
	//CONSTRUCTOR
	public BloqueAudio()
	{
		idEstacionMonitorizada=-1;	
	}
	
	//mtodo que establece la estacin a monitorizar con mensajes de audio
	public void setEstacionMonitorizada(int idEstacionMonitorizada) 
	{
		this.idEstacionMonitorizada=idEstacionMonitorizada;
	}
	
	//Mtodo que elimina la monitorizacin de la estacin actual
	public void unsetEstacionMonitorizada() 
	{
		this.idEstacionMonitorizada=-1;
	}
	
	//Mtodo que reproduce el sonido indicado
	public void reproducir(String sonido)
	{
		try
		{
			if(ac!=null) ac.stop();
			URL resource=getClass().getClassLoader().getResource(sonido);
			if (resource!=null) 
			{
				ac=Applet.newAudioClip(resource);
				ac.play();
			}
		}catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	//Mtodo que procesa los eventos relativos a la generacin de datos por parte de los Usuarios	
	public void notificarDatosGenerados(int idslot, int idevento, int idEstacion, int evento, Datos datos)
	{
		if(idevento==-1) return;
		if (idEstacion==idEstacionMonitorizada)
			reproducir(factoria.getSonido(evento));

	}
	
	//Mtodo que procesa los eventos relativos a la aceptacin o rechazo de los datos enviados por el usuario a los Controles de Acceso
	public void notificarRecepcionDeDatos(int idslot, int idevento, int idEstacion, int estadoRecepcion, Datos datos)
	{
		if(idevento==-1) return;
		if (idEstacion==idEstacionMonitorizada)
		{
			if(factoria.getSonido(estadoRecepcion)!=null)
				reproducir(factoria.getSonido(estadoRecepcion));
		}
	}
	
	//Mtodo que procesa los eventos relativos a la recepcin de celdas
	public void notificarCeldasRecibidas(int version, int idslot, int idevento, int idEstacion, Celda celdaEntradaBus1, Celda celdaSalidaBus1, Celda celdaEntradaBus2, Celda celdaSalidaBus2, boolean incrementaVacias1, boolean incrementaVacias2)
	{

		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
	
		if (celdaEntradaBus1!=null && celdaEntradaBus2!=null)
		{
			if(celdaSalidaBus1==null && celdaSalidaBus2==null) reproducir(factoria.getSonido(Estacion.CA1_Y_CA2_RECIBEN_NOENVIAN));
			if(celdaSalidaBus1==null && celdaSalidaBus2!=null) reproducir(factoria.getSonido(Estacion.CA1_RECIBE_NOENVIA_CA2_RECIBE_ENVIA));
			if(celdaSalidaBus1!=null && celdaSalidaBus2==null) reproducir(factoria.getSonido(Estacion.CA1_RECIBE_ENVIA_CA2_RECIBE_NOENVIA));
			if(celdaSalidaBus1!=null && celdaSalidaBus2!=null) reproducir(factoria.getSonido(Estacion.CA1_Y_CA2_RECIBEN_ENVIAN));
		}
		
		if (celdaEntradaBus1!=null && celdaEntradaBus2==null)
		{
			if(celdaSalidaBus1!=null) reproducir(factoria.getSonido(Estacion.CA1_RECIBE_ENVIA_CA2_NO_RECIBE));
			else reproducir(factoria.getSonido(Estacion.CA1_RECIBE_NOENVIA_CA2_NO_RECIBE));
		}
		if (celdaEntradaBus1==null && celdaEntradaBus2!=null)
		{
			if(celdaSalidaBus2!=null) reproducir(factoria.getSonido(Estacion.CA1_NORECIBE_CA2_RECIBE_ENVIA));
			else reproducir(factoria.getSonido(Estacion.CA1_NORECIBE_CA2_RECIBE_NOENVIA));
		}
	}
	
	//Mtodo que procesa los eventos relativos al envo de datos desde los Controles de acceso a los Usuarios
	public void notificarContenidoCeldas(int idslot, int idevento, int idEstacion, int estadoCelda1, int estadoCelda2)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
		
		if(estadoCelda1==Estacion.CA1_ENVIA_DATOS_A_USUARIO && estadoCelda2==Estacion.CA2_ENVIA_DATOS_A_USUARIO)
			reproducir(factoria.getSonido(Estacion.CA1_CA2_A_USUARIO));
		else
		{
			if(estadoCelda1==Estacion.CA1_ENVIA_DATOS_A_USUARIO) reproducir(factoria.getSonido(estadoCelda1));
			if(estadoCelda2==Estacion.CA2_ENVIA_DATOS_A_USUARIO) reproducir(factoria.getSonido(estadoCelda2));
		}
	}
	
	//Mtodo que procesa los eventos relativos a la insercin de datos en celdas
	public void notificarInsercionDeDatosPendientes(int idslot, int idevento, int idEstacion, int estadoDatos1, int estadoDatos2)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
	
		if(estadoDatos1==Estacion.CA1_INSERTA_DATOS && estadoDatos2==Estacion.CA2_INSERTA_DATOS) reproducir(factoria.getSonido(Estacion.CA1_CA2_INSERTAN_DATOS));
		else if(estadoDatos1==Estacion.CA1_INSERTA_DATOS) reproducir(factoria.getSonido(estadoDatos1));
		else if(estadoDatos2==Estacion.CA2_INSERTA_DATOS) reproducir(factoria.getSonido(estadoDatos2));
		else if(estadoDatos1==Estacion.CA1_INSERTA_DATOS_V3V4 && estadoDatos2==Estacion.CA2_INSERTA_DATOS_V3V4) reproducir(factoria.getSonido(Estacion.CA1_CA2_INSERTAN_DATOS_V3V4));
		else if(estadoDatos1==Estacion.CA1_INSERTA_DATOS_V3V4) reproducir(factoria.getSonido(estadoDatos1));
		else if(estadoDatos2==Estacion.CA2_INSERTA_DATOS_V3V4) reproducir(factoria.getSonido(estadoDatos2));
		

	}
	
	//Mtodo que procesa los eventos relativos al borrado de celdas
	public void notificarBorradoCelda(int idslot, int idevento, int idEstacion, int estadoCA1, int estadoCA2)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
		if (estadoCA1==Estacion.CA1_BORRA_DATOS && estadoCA2==Estacion.CA2_BORRA_DATOS) 
			reproducir(factoria.getSonido(Estacion.CA1_CA2_BORRAN_DATOS));
		
		else if(estadoCA1==Estacion.CA1_BORRA_DATOS) reproducir(factoria.getSonido(estadoCA1));
		else if(estadoCA2==Estacion.CA2_BORRA_DATOS) reproducir(factoria.getSonido(estadoCA2));
		else if(estadoCA1==Estacion.CA1_NO_DESTINATARIO) reproducir(factoria.getSonido(estadoCA1));
		else if(estadoCA2==Estacion.CA2_NO_DESTINATARIO) reproducir(factoria.getSonido(estadoCA2));
		
	}
	
	//Mtodo que procesa los eventos relativos a las solicitudes de envo de peticiones "ro arriba"
	public void notificarSolicitudEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
		
		if(estadoPeticion1==Estacion.CA1_SOLICITA_ENVIO_DE_PETICION) reproducir(factoria.getSonido(estadoPeticion1));
		else if(estadoPeticion2==Estacion.CA2_SOLICITA_ENVIO_DE_PETICION) reproducir(factoria.getSonido(estadoPeticion2));
		
	}
	
	//Mtodo que procesa los eventos relativos al envo de las seales RC
	public void notificarEnvioRC(int idslot, int idevento, int idEstacion, int estadoRC1, int estadoRC2, int[] contadorPeticiones, int[] contadorVacias, int[] cuentaAtras)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
		
		if(estadoRC1==Estacion.CA1_ENVIA_RC) reproducir(factoria.getSonido(estadoRC1));
		else if(estadoRC2==Estacion.CA2_ENVIA_RC) reproducir(factoria.getSonido(estadoRC2));
	}
	
	//Mtodo que procesa los eventos relativos a la insercin de peticiones en las celdas
	public void notificarEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		if(idevento==-1) return;
		if (idEstacion!=idEstacionMonitorizada) return;
		
		if(estadoPeticion1==Estacion.CA1_ENVIA_PETICION && estadoPeticion2==Estacion.CA2_ENVIA_PETICION) reproducir(factoria.getSonido(Estacion.CA1_CA2_ENVIAN_PETICION));
		else if(estadoPeticion1==Estacion.CA1_ENVIA_PETICION) reproducir(factoria.getSonido(estadoPeticion1));
		else if(estadoPeticion2==Estacion.CA2_ENVIA_PETICION) reproducir(factoria.getSonido(estadoPeticion2));
	}
	
	//Mtodo que prepara al observador para iniciar una nueva simulacin (resetea la estacin monitorizada)
	public void notificarNuevaSimulacion(int version, int numeroEstaciones,  double pgUnica, double[] arrayPg, Estacion[] estaciones)
	{
		idEstacionMonitorizada=-1;
	}
}


