import java.util.*;
import java.awt.Color;

//CLASE ABSTRACTA QUE DEFINE EL BLOQUE ESTACIN. TODAS LAS VERSIONES EXTENDERN DE ESTA CLASE, SOBRESCRIBIENDO LOS MTODOS QUE LES INTERESE
public abstract class Estacion
{	
	protected Vector<Integer> estado;
	protected Vector<Integer> eventoActual;
	protected Celda[] celdaEntrada, celdaSalida;
	protected Datos[] datosPendientes;
	protected Datos datosGeneradosPorUsuario;
	protected boolean[] solicitarPet;
	protected boolean[] enviarRc;
	protected boolean[] peticionPendiente;
	protected int[] cuentaAtras, contadorVacias, contadorPeticiones;	
	protected int idEstacion;
	protected int numeroEstaciones;
	protected long semilla;
	protected Random random;
	protected double pg;
	protected int version, checkpoint;
	protected int visibilidad;
	protected boolean explicar;
	protected int[] datosCursados, tiempoDeServicio, datosAceptados, datosRechazados;
	protected int datosGeneradosUsuario;
	protected int[] peticionesPendientes;
	
	
	//CAMPOS ESTTICOS QUE REPRESENTAN ESTADOS DE LOS COMPONENTES DE LA ESTACIN
	public static final int USUARIO_GENERA_DATOS_CA1=1;
	public static final int USUARIO_GENERA_DATOS_CA2=2;
	public static final int USUARIO_NO_GENERA_DATOS=3;
	public static final int CA1_ACEPTA_DATOS=4;
	public static final int CA2_ACEPTA_DATOS=5;
	public static final int CA1_MARCA_TX_PENDIENTE=6;
	public static final int CA2_MARCA_TX_PENDIENTE=7;
	public static final int USUARIO_NO_HA_ENVIADO_DATOS=8;
	public static final int CA1_RECHAZA_DATOS=9;
	public static final int CA2_RECHAZA_DATOS=10;
	public static final int CA1_RECIBIDA_CELDA_VACIA=11;
	public static final int CA2_RECIBIDA_CELDA_VACIA=12;
	public static final int CA1_RECIBIDA_CELDA_OCUPADA=13;
	public static final int CA2_RECIBIDA_CELDA_OCUPADA=14;
	public static final int CA1_CELDA_NO_RECIBIDA=15;
	public static final int CA2_CELDA_NO_RECIBIDA=16;

	public static final int CA1_Y_CA2_NO_RECIBEN_CELDA=17;
	public static final int CA1_NORECIBE_CA2_RECIBE_ENVIA=18;
	public static final int CA1_NORECIBE_CA2_RECIBE_NOENVIA=19;
	public static final int CA1_RECIBE_ENVIA_CA2_NO_RECIBE=20;
	public static final int CA1_RECIBE_NOENVIA_CA2_NO_RECIBE=21;
	public static final int CA1_RECIBE_ENVIA_CA2_RECIBE_NOENVIA=22;
	public static final int CA1_RECIBE_NOENVIA_CA2_RECIBE_ENVIA=23;
	public static final int CA1_Y_CA2_RECIBEN_ENVIAN=24;
	public static final int CA1_Y_CA2_RECIBEN_NOENVIAN=25;

	public static final int CA1_ENVIA_DATOS_A_USUARIO=26;
	public static final int CA2_ENVIA_DATOS_A_USUARIO=27;
	public static final int CA1_NO_TIENE_CELDA=28;
	public static final int CA2_NO_TIENE_CELDA=29;
	
	public static final int CA1_CA2_A_USUARIO=30;
	public static final int CA1_A_USUARIO_CA2_CELDA_NO_RECIBIDA=31;
	public static final int CA1_A_USUARIO_CA2_CELDA_VACIA=32;
	public static final int CA1_CELDA_NO_RECIBIDA_CA2_A_USUARIO=33;
	public static final int CA1_CA2_CELDAS_NO_RECIBIDAS=34;
	public static final int CA1_CELDA_NO_RECIBIDA_CA2_CELDA_VACIA=35;
	public static final int CA1_CELDA_VACIA_CA2_A_USUARIO=36;
	public static final int CA1_CELDA_VACIA_CA2_CELDA_NO_RECIBIDA=37;
	public static final int CA1_CA2_CELDAS_VACIAS=38;
	
	public static final int CA1_INSERTA_DATOS=39;
	public static final int CA2_INSERTA_DATOS=40;
	public static final int CA1_CA2_INSERTAN_DATOS=41;
	
	public static final int CA1_INSERTA_DATOS_V3V4=42;
	public static final int CA2_INSERTA_DATOS_V3V4=43;
	public static final int CA1_CA2_INSERTAN_DATOS_V3V4=44;
	
	public static final int CA1_NO_PUEDE_INSERTAR_DATOS=45;
	public static final int CA2_NO_PUEDE_INSERTAR_DATOS=46;
	public static final int CA1_NO_TIENE_DATOS_PENDIENTES=47;
	public static final int CA2_NO_TIENE_DATOS_PENDIENTES=48;
	
	public static final int CA1_BORRA_DATOS=49;
	public static final int CA2_BORRA_DATOS=50;
	public static final int CA1_CA2_BORRAN_DATOS=51;
	public static final int CA1_NO_DESTINATARIO=52;
	public static final int CA2_NO_DESTINATARIO=53;
	
	public static final int CA1_SOLICITA_ENVIO_DE_PETICION=54;
	public static final int CA2_SOLICITA_ENVIO_DE_PETICION=55;
	public static final int CA1_NO_SOLICITA_ENVIO_DE_PETICION=56;
	public static final int CA2_NO_SOLICITA_ENVIO_DE_PETICION=57;
	
	public static final int CA1_ENVIA_RC=58;
	public static final int CA2_ENVIA_RC=59;
	public static final int CA1_NO_ENVIA_RC=60;
	public static final int CA2_NO_ENVIA_RC=61;
	
	public static final int CA1_NO_PUEDE_ENVIAR_PETICION=62;
	public static final int CA2_NO_PUEDE_ENVIAR_PETICION=63;		
	public static final int CA1_ENVIA_PETICION=64;
	public static final int CA2_ENVIA_PETICION=65;	
	public static final int CA1_CA2_ENVIAN_PETICION=66;
	public static final int CA1_NO_TIENE_PETICION_PENDIENTE=67;
	public static final int CA2_NO_TIENE_PETICION_PENDIENTE=68;
	
	//CONSTUCTOR
	public Estacion(int idEstacion, double pg, int numeroEstaciones, int version)
	{
	
		estado=new Vector<Integer>();
		this.version=version;
		celdaEntrada=new Celda[2];
		celdaSalida=new Celda[2];
		datosPendientes=new Datos[2];
		datosCursados=new int[2];
		tiempoDeServicio=new int[2];
		datosAceptados=new int[2];
		datosRechazados=new int[2];
		solicitarPet=new boolean[2];
		enviarRc=new boolean[2];
		cuentaAtras= new int[2];
		contadorVacias= new int[2];
		contadorPeticiones= new int[2];
		peticionPendiente=new boolean[2];
		peticionesPendientes=new int[2];
		this.idEstacion=idEstacion;
		this.numeroEstaciones=numeroEstaciones;
		this.pg=pg;
		this.semilla=(int) (Math.random()*2000);

		random=new Random(semilla);
		random.nextDouble();
	}
	
	//MTODOS IMPLEMENTADOS:
	public void establecerCeldasDeSalida() //TODAS LAS VERSIONES
	{
		celdaSalida[0]=(celdaEntrada[0]!=null? celdaEntrada[0].clonar(): null);
		celdaSalida[1]=(celdaEntrada[1]!=null? celdaEntrada[1].clonar(): null);
	}
	
	public void setCeldaEntrada(int idBus, Celda celda) //TODAS LAS VERSIONES
	{
		estado.removeAllElements();
		celdaEntrada[idBus-1]=celda;	
	}
	
	public void setCeldaSalida(int idBus, Celda celda)  //TODAS LAS VERSIONES
	{
		celdaSalida[idBus-1]=celda;
	}

	public Celda getCeldaEntrada(int idBus)  //TODAS LAS VERSIONES
	{
		return celdaEntrada[idBus-1];
	}
	
	public Celda getCeldaSalida(int idBus)  //TODAS LAS VERSIONES
	{
		return celdaSalida[idBus-1];
	}
	
	public void setPg(double pg)  //TODAS LAS VERSIONES
	{
		this.pg=pg;
	}
	
	public double getPg()  //TODAS LAS VERSIONES
	{
		return pg;
	}
	
	public Datos getDatosGeneradosPorUsuario()  //TODAS LAS VERSIONES
	{
		return datosGeneradosPorUsuario;
	}	

	public int comprobarSiUsuarioGeneraDatos()  //TODAS LAS VERSIONES
	{		
		int nuevoEstado;

		//Comprobamos si genera datos:
		if(GeneradorAleatorio.nextDouble()<pg)
		{	
			int destino;
			
			//Genera un nmero entero aleatorio entre 1 y numeroDeEstaciones, ambos incluidos excluyendo el nmero de esta estacin (no puede enviarse datos a s misma)
			do
			{
				destino=((int) (GeneradorAleatorio.nextDouble()*numeroEstaciones)) +1;
			}
			while (destino==idEstacion); //Inicialmente destino=ID luego siempre que quiera transmitir entrar al menos una vez. En caso de que el nmero de estacin elegido aleatoriamente  sea su propio nmero entonces volver a repetir la operacin, slo saldr del bucle cuando el nmero elegido como destino sea distinto del suyo
			
			
			datosGeneradosPorUsuario=new Datos(idEstacion, destino);
			datosGeneradosUsuario++;
			
	
			nuevoEstado=(datosGeneradosPorUsuario.getDestino()>idEstacion? USUARIO_GENERA_DATOS_CA1: USUARIO_GENERA_DATOS_CA2); 		

		}
		//Si no genera datos:
		else
		{
			datosGeneradosPorUsuario=null;
			nuevoEstado=USUARIO_NO_GENERA_DATOS;
		}

		estado.add(nuevoEstado);
		return nuevoEstado;
	}
	
	
	public Datos getDatosPendientes(int idBus)  //TODAS LAS VERSIONES
	{
		return datosPendientes[idBus-1];
	}

	public int getDatosAceptados(int idBus) //TODAS LAS VERSIONES
	{
		return datosAceptados[idBus-1];
	}
	
	public int getDatosRechazados(int idBus) //TODAS LAS VERSIONES
	{
		return datosRechazados[idBus-1];
	}
	
	public int getDatosCursados(int idBus) //TODAS LAS VERSIONES
	{
		return datosCursados[idBus-1];
	}
	
	public int getDatosGeneradosUsuario() //TODAS LAS VERSIONES
	{
		return datosGeneradosUsuario;
	}
	
	public Vector<Integer> getEstado() //TODAS LAS VERSIONES
	{
		return estado;
	}
	
	public void incrementarDatosAceptados(int idBus) //TODAS LAS VERSIONES
	{
		datosAceptados[idBus]=datosAceptados[idBus]+1;
	}
	
	public void incrementarDatosRechazados(int idBus) //TODAS LAS VERSIONES
	{
		datosRechazados[idBus]=datosRechazados[idBus]+1;
	}
	
	public void incrementarDatosCursados(int idBus) //TODAS LAS VERSIONES
	{
		datosCursados[idBus]=datosCursados[idBus]+1;
	}
	
	public void incrementarTiempoDeServicio(int idBus) //TODAS LAS VERSIONES
	{
		tiempoDeServicio[idBus]=tiempoDeServicio[idBus]+1;
	}
	 
	public void incrementarTiempoDeServicio(int idBus, int incremento) //TODAS LAS VERSIONES
	{
		tiempoDeServicio[idBus]=tiempoDeServicio[idBus]+incremento;	
	}
	
	public int getTraficoCursado(int idBus) //TODAS LAS VERSIONES
	{
		return datosCursados[idBus-1];
	}
	
	public int getTiempoDeServicio(int idBus) //TODAS LAS VERSIONES
	{
		return tiempoDeServicio[idBus-1];
	}
	
	
	public void resetear() //TODAS LAS VERSIONES
	{
		celdaEntrada=new Celda[2];
		celdaSalida=new Celda[2];
		datosPendientes=new Datos[2];
		solicitarPet=new boolean[2];
		enviarRc=new boolean[2];
		cuentaAtras= new int[2];
		contadorVacias= new int[2];
		contadorPeticiones= new int[2];
		peticionPendiente=new boolean[2];
	
		datosCursados=new int[2];
		tiempoDeServicio=new int[2];
		datosAceptados=new int[2];
		datosRechazados=new int[2];
	
	}
	
	
	
	public int getPeticionesPendientes(int idBus) //VERSIONES 3 y 4
	{
		return peticionesPendientes[idBus-1];
	}
	
	public void incrementarContadorPeticiones(int idBus) //VERSIONES 3 y 4
	{
		contadorPeticiones[idBus-1]=contadorPeticiones[idBus-1]+1;
	}
	
	public void incrementarContadorVacias(int idBus)  //VERSIONES 3 y 4
	{
		contadorVacias[idBus-1]=contadorVacias[idBus-1]+1;
	}
	
	public int[] getContadorVacias() //VERSIONES 3 y 4
	{
		return contadorVacias;
	}
	
	public int[] getContadorPeticiones() //VERSIONES 3 y 4
	{
		return contadorPeticiones;
	}
	
	public int[] getCuentaAtras() //VERSIONES 3 y 4
	{
		return cuentaAtras;
	}
	
	public int getContadorVacias(int idBus) //VERSIONES 3 y 4
	{
		return contadorVacias[idBus-1];
	}
	
	public int getContadorPeticiones(int idBus) //VERSIONES 3 y 4
	{
		return contadorPeticiones[idBus-1];
	}
	
	public int getCuentaAtras(int idBus) //VERSIONES 3 y 4
	{
		return cuentaAtras[idBus-1];
	}
	
	//MTODOS ABSTRACTOS
	public abstract int comprobarSiDatosSonAceptados(); //TODAS LAS VERSIONES
	public abstract int comprobarSiCeldaContieneDatos(int idBus); //TODAS LAS VERSIONES
	public abstract int comprobarDatosPendientesCA(int idBus); //TODAS LAS VERSIONES
	
	public abstract boolean contabilizarCeldaVacia(int idBus); //VERSIONES 3 y 4
	public abstract int borrarCelda(int idBus); //VERSIONES 2 y 4	
	public abstract int comprobarSolicitudEnvioDePeticion(int idBus); //VERSIONES 3 y 4
	public abstract int comprobarEnvioDePeticion(int idBus); //VERSIONES 3 y 4
	public abstract int comprobarEnvioRC(int idBus); //VERSIONES 3 y 4
	
	
	

	

	

}















