import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

//CLASE QUE IMPLEMENTA EL PANEL POPUP QUE APARECE AL HACER CLIC DERECHO SOBRE UNA ESTACIN, PERMITIENDO CAMBIAR LA PG DE LA MISMA Y VER ESTADSTICAS RECOPILADAS DURANTE LA SIMULACIN
public class PanelConfiguracionDeEstacion extends JPopupMenu implements ActionListener
{
	private Estacion estacion;
	private BloqueGrafico bloqueGrafico;
	private TextFieldNumerico textFieldPg;
	private JLabel labelPg, avisos;
	private JButton botonCambiar;
	private int idEstacion;
	private double pg;
	
	//CONSTRUCTOR
	public PanelConfiguracionDeEstacion(BloqueGrafico bloqueGrafico, int idEstacion, Estacion estacion)
	{
		super();

		this.estacion=estacion;
		
		setBorder(BorderFactory.createLineBorder(Color.black, 2));
		
		Boton botonCambiar=new Boton(CargadorDeImagenes.getCargador().getIcono("botonCambiar.gif"), CargadorDeImagenes.getCargador().getIcono("botonCambiarPressed.gif"));
		botonCambiar.addActionListener(this);
		
		this.bloqueGrafico=bloqueGrafico;

		this.idEstacion=idEstacion;
		
		
		PanelEstadisticas contenedor=new PanelEstadisticas("panelEstadisticas3.gif", idEstacion, estacion);
		setSize(contenedor.getPreferredSize());

		setOpaque(false);

		textFieldPg=new TextFieldNumerico(2, 5);
		textFieldPg.setFont(new Font("Arial", Font.BOLD, 22));
		textFieldPg.setText(""+estacion.getPg());
		Dimension dim=new Dimension(90,32);
		textFieldPg.setPreferredSize(dim);
		textFieldPg.setMaximumSize(dim);
		textFieldPg.setMinimumSize(dim);
	
		JPanel panelAuxiliar=new JPanel();
		panelAuxiliar.setLayout(new BoxLayout(panelAuxiliar, BoxLayout.X_AXIS));
		panelAuxiliar.setOpaque(false);
		panelAuxiliar.add(Box.createRigidArea(new Dimension(60,0)));
		panelAuxiliar.add(textFieldPg);
		panelAuxiliar.add(Box.createRigidArea(new Dimension(10,0)));
		panelAuxiliar.add(botonCambiar);
		
		avisos=new JLabel("");
		contenedor.setOpaque(false);
		contenedor.setLayout(new BoxLayout(contenedor, BoxLayout.Y_AXIS));
		contenedor.add(Box.createRigidArea(new Dimension(0,32)));
		contenedor.add(panelAuxiliar);
		contenedor.add(avisos);
		add(contenedor);
		
		pack();	

	}
	
	//Mtodo de la interfaz ActionListener: Se activa al pulsar el botn "botonCambiar" y comprueba que la PG introducida sea vlida, asignndosela a la estacin en caso afirmativo
	public void actionPerformed(ActionEvent ev)
	{
		String StringPg=textFieldPg.getText();
		if((StringPg.compareTo("")==0) || !verificarPg(pg)) avisos.setText("PG ERRNEA"); 
		else
		{
			double pg=Double.parseDouble(StringPg); 
			if (verificarPg(pg))
			{
				estacion.setPg(pg);
				setVisible(false);
			}
			
		}
	}
	
	//Verifica que el double corresponda a una probabilidad (es decir, que este comprendido entre 0.0 y 1.0, ambos inclusive)
	public boolean verificarPg(double pg)
	{
		if (pg>=0.0 && pg<=1.0) return true;		
		return false;		
	}
	
}



//CLASE AUXILIAR EMPLEADA POR PanelConfiguracionDeEstacion PARA PINTAR LOS DATOS ESTADSTICOS DE LA ESTACIN
class PanelEstadisticas extends PanelConFondo implements MouseListener
{
	private boolean ayudaActivada;
	private int idEstacion;
	private int usuarioDatosGenerados, usuarioDatosEnviados, usuarioDatosPerdidos;
	private int CASuperiorDatosCursados, CASuperiorDatosRechazados;
	private int CAInferiorDatosCursados, CAInferiorDatosRechazados;
	private String usuarioPorcentajeDatosEnviados, usuarioPorcentajeDatosPerdidos;
	private String CASuperiorPorcentajeDatosCursados, CASuperiorPorcentajeDatosRechazados;
	private String CAInferiorPorcentajeDatosCursados, CAInferiorPorcentajeDatosRechazados;
	private String CASuperiorTiempoServicio, CAInferiorTiempoServicio;
	//CONSTRUCTOR
	public PanelEstadisticas(String fondo, int idEstacion, Estacion estacion)
	{
		super(fondo);
		addMouseListener(this);
		this.idEstacion=idEstacion;
		
		//CLCULO DE ESTADSTICAS:
		this.usuarioDatosGenerados=estacion.getDatosGeneradosUsuario();
		this.usuarioDatosEnviados=this.usuarioDatosGenerados-estacion.getDatosRechazados(1)-estacion.getDatosRechazados(2);
		this.usuarioDatosPerdidos=estacion.getDatosRechazados(1)+estacion.getDatosRechazados(2);
		
		this.CASuperiorDatosCursados=estacion.getDatosCursados(1);
		this.CASuperiorDatosRechazados=estacion.getDatosRechazados(1);
		CASuperiorTiempoServicio=(CASuperiorDatosCursados==0?"-":""+((double) estacion.getTiempoDeServicio(1)/(double) CASuperiorDatosCursados)); 
		
		this.CAInferiorDatosCursados=estacion.getDatosCursados(2);
		this.CAInferiorDatosRechazados=estacion.getDatosRechazados(2);
		CAInferiorTiempoServicio=(CAInferiorDatosCursados==0?"-":""+((double) estacion.getTiempoDeServicio(2)/(double) CAInferiorDatosCursados));

		usuarioPorcentajeDatosEnviados=(usuarioDatosGenerados==0? "":""+((double) (usuarioDatosEnviados*100)/(double)usuarioDatosGenerados));
		usuarioPorcentajeDatosPerdidos=(usuarioDatosGenerados==0? "":""+((double)(usuarioDatosPerdidos*100)/(double)usuarioDatosGenerados));
		
		CASuperiorPorcentajeDatosCursados=(CASuperiorDatosCursados+CASuperiorDatosRechazados==0? "":""+((double)(CASuperiorDatosCursados*100)/(double)(CASuperiorDatosCursados+CASuperiorDatosRechazados)));
		CASuperiorPorcentajeDatosRechazados=(CASuperiorDatosCursados+CASuperiorDatosRechazados==0? "": ""+((double)(CASuperiorDatosRechazados*100)/(double)(CASuperiorDatosCursados+CASuperiorDatosRechazados)));
		CAInferiorPorcentajeDatosCursados=(CAInferiorDatosCursados+CAInferiorDatosRechazados==0?"":""+((double)(CAInferiorDatosCursados*100)/(double)(CAInferiorDatosCursados+CAInferiorDatosRechazados)));
		CAInferiorPorcentajeDatosRechazados=(CAInferiorDatosCursados+CAInferiorDatosRechazados==0?"":""+((double)(CAInferiorDatosRechazados*100)/(double)(CAInferiorDatosCursados+CAInferiorDatosRechazados)));
		
		usuarioPorcentajeDatosEnviados=usuarioPorcentajeDatosEnviados.substring(0, Math.min(usuarioPorcentajeDatosEnviados.length(), 5));
		usuarioPorcentajeDatosPerdidos=usuarioPorcentajeDatosPerdidos.substring(0, Math.min(usuarioPorcentajeDatosPerdidos.length(), 5));
		CASuperiorPorcentajeDatosCursados=CASuperiorPorcentajeDatosCursados.substring(0, Math.min(CASuperiorPorcentajeDatosCursados.length(), 5));
		CASuperiorPorcentajeDatosRechazados=CASuperiorPorcentajeDatosRechazados.substring(0, Math.min(CASuperiorPorcentajeDatosRechazados.length(), 5));
		CAInferiorPorcentajeDatosCursados=CAInferiorPorcentajeDatosCursados.substring(0, Math.min(CAInferiorPorcentajeDatosCursados.length(), 5));
		CAInferiorPorcentajeDatosRechazados=CAInferiorPorcentajeDatosRechazados.substring(0, Math.min(CAInferiorPorcentajeDatosRechazados.length(), 5));
		CASuperiorTiempoServicio=CASuperiorTiempoServicio.substring(0, Math.min(CASuperiorTiempoServicio.length(), 5));
		CAInferiorTiempoServicio=CAInferiorTiempoServicio.substring(0, Math.min(CAInferiorTiempoServicio.length(), 5));

		
	}
	
	//Sobreescribe el modopaintComponent para mostrar los datos estadsticos
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		
		//Escribimos y suavizamos el nmero de estacin y las estadsticas
		Graphics2D graphics2d = (Graphics2D)g;
		graphics2d.setColor(Color.red);
		graphics2d.setFont(new Font("Arial", Font.BOLD, 26));
		graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		graphics2d.drawString(""+idEstacion, 150, 26);
		
		graphics2d.setColor(Color.black);
		graphics2d.setFont(new Font("Arial", Font.BOLD, 10));
		graphics2d.drawString("CLIC para ayuda", 182, 85);
		
		//Si la ayuda est desactivada muestra las estadsticas	
		if (!ayudaActivada)
		{
			graphics2d.setFont(new Font("Arial", Font.BOLD, 14));
			graphics2d.drawString(""+usuarioDatosGenerados,146,134);
			graphics2d.drawString(""+usuarioDatosEnviados+" ("+usuarioPorcentajeDatosEnviados+"%)",169,147);
			graphics2d.drawString(""+usuarioDatosPerdidos+" ("+usuarioPorcentajeDatosPerdidos+"%)",169,160);
			graphics2d.drawString(""+CASuperiorDatosCursados+" ("+CASuperiorPorcentajeDatosCursados+"%)",152,210);
			graphics2d.drawString(""+CASuperiorDatosRechazados+" ("+CASuperiorPorcentajeDatosRechazados+"%)",152,223);
			graphics2d.drawString(""+CASuperiorTiempoServicio+" slots",152,236);
			graphics2d.drawString(""+CAInferiorDatosCursados+" ("+CAInferiorPorcentajeDatosCursados+"%)",152,286);
			graphics2d.drawString(""+CAInferiorDatosRechazados+" ("+CAInferiorPorcentajeDatosRechazados+"%)",152,299);
			graphics2d.drawString(""+CAInferiorTiempoServicio+" slots",152,312);
		}
		
		//Si la ayuda est activada muestra la ayuda	
		else g.drawImage(CargadorDeImagenes.getCargador().getImagen("ayudaEstadisicas.gif"),133, 94, this);
		
	}
	
	//Mtodos de la interfaz MouseListener
	public void mousePressed(MouseEvent ev)
	{
		//Si el usuario pulsa el botn sobre este panel se activa la ayuda
		ayudaActivada=true;
		repaint();
	}
	public void mouseReleased(MouseEvent ev)
	{
		//Si el usuario suelta el botn se activa la ayuda
		ayudaActivada=false;
		repaint();
	}
	public void mouseClicked(MouseEvent ev){}
	public void mouseEntered(MouseEvent ev){}
	public void mouseExited(MouseEvent ev){}
}