import java.util.*;
import java.awt.Color;

//CLASE QUE IMPLEMENTA LA ESTACIN DE LA VERSIN 3 (IMPLEMENTA EL MECANISMO DE COLA DISTRIBUIDA)
public class EstacionVersion3 extends Estacion
{	

	//CONSTRUCTOR
	public EstacionVersion3(int idEstacion, double pg, int numeroEstaciones, int version)
	{
		super(idEstacion, pg, numeroEstaciones, version);

	}

	//Sobreescribe el mtodo comprobarSiDatosSonAceptados de la clase abstracta Estacion
	public int comprobarSiDatosSonAceptados()
	{	
		int nuevoEstado;
		
		//Si el usuario no gener datos:
		if(datosGeneradosPorUsuario==null) return USUARIO_NO_HA_ENVIADO_DATOS;

		//Si el usuario s gener datos se determina el destino de los mismos
		int destino=datosGeneradosPorUsuario.getDestino();
	
		int idBus=(destino>idEstacion? 0: 1);
	
	
		//Si el Control de Acceso destino no tiene datos pendientes de envo acepta los datos	
		if(datosPendientes[idBus]==null)
		{
			datosPendientes[idBus]=datosGeneradosPorUsuario;
			incrementarDatosAceptados(idBus);
			solicitarPet[idBus]=true; //Activa el flag enviarPet para que en el prximo evento se enve la solicitud de envo de peticin ro arriba
			nuevoEstado=(idBus==0? CA1_ACEPTA_DATOS: CA2_ACEPTA_DATOS);
		}
		
		//Si el Control de Acceso destino tiene datos pendientes de envo rechaza los datos	
		else
		{
			incrementarDatosRechazados(idBus);
			nuevoEstado=(idBus==0? CA1_RECHAZA_DATOS: CA2_RECHAZA_DATOS);
		}

		estado.add(nuevoEstado);
		return nuevoEstado;

	}
	
	//Sobreescribe el mtodo comprobarSiCeldaContieneDatos de la clase abstracta Estacion
	public int comprobarSiCeldaContieneDatos(int idBus)
	{
		//Establecemos las celdas de entrada como celdas de salida, para enviarlas la prxima vez que se reciba una celda
		if(idBus==1) establecerCeldasDeSalida(); 

		//Si la celda de salida es null significar que no se recibi celda alguna
		if (celdaSalida[idBus-1]==null) 
		{
			return (idBus==1? CA1_CELDA_NO_RECIBIDA: CA2_CELDA_NO_RECIBIDA); //No se ha recibido celda (caso inicial hasta qe la primera celda llegue al terminador destino)	
		}
		
		//Si la celda de salida contiene datos y no han sido introducidos por esta estacin se envan lso datos al usuario
		if (celdaSalida[idBus-1].getOcupada() && celdaSalida[idBus-1].getDatos().getOrigen()!=idEstacion) 
		{
			int nuevoEstado=(idBus==1? CA1_ENVIA_DATOS_A_USUARIO: CA2_ENVIA_DATOS_A_USUARIO);
			estado.add(nuevoEstado);
			return nuevoEstado;
		}
			
		//Si se recibi una celda pero no contena datos entonces est vaca
		return (idBus==1? CA1_RECIBIDA_CELDA_VACIA: CA2_RECIBIDA_CELDA_VACIA);
	}
	
	
	//Sobreescribe el mtodo contabilizarCeldaVacia de la clase abstracta Estacion
	public boolean contabilizarCeldaVacia(int idBus) //Cuando se enve una celda vaca a la siguiente estacin se actualizar CD o bien contadorVacias
	{
		//Si el contador CD es mayor que cero ser decrementado en una unidad
		if (cuentaAtras[idBus-1]>0)
		{
			cuentaAtras[idBus-1]=cuentaAtras[idBus-1]-1; 
			return false;
		}
		
		//Si el contador CD no era mayor que cero se incrementar el contador de vacas
		contadorVacias[idBus-1]= contadorVacias[idBus-1]+1;
		return true;	
				
		
	}
	
	//Sobreescribe el mtodo comprobarDatosPendientesCA de la clase abstracta Estacion
	public int comprobarDatosPendientesCA(int idBus)
	{
		Datos datosPendientes=this.datosPendientes[idBus-1];
		
		//Si no hay celda de salida:
		if (celdaSalida[idBus-1]==null) 
		{
			if(datosPendientes!=null) incrementarTiempoDeServicio(idBus-1);
			return (idBus==1? CA1_NO_TIENE_CELDA: CA2_NO_TIENE_CELDA);
		}
	
		//Si hay celda de salida y el CA tiene datos pendientes
		if (datosPendientes!=null)
		{	
			//Si la celda est vaca y el contador CD es menor o igual que cero inserta los datos
			if (!celdaSalida[idBus-1].getOcupada())
			{
				if (cuentaAtras[idBus-1]<=0)
				{
					celdaSalida[idBus-1].setDatos(datosPendientes);
					this.datosPendientes[idBus-1]=null;
					
					incrementarDatosCursados(idBus-1);
					return (idBus==1? CA1_INSERTA_DATOS_V3V4: CA2_INSERTA_DATOS_V3V4);
				}
			}
			
			//Si haba datos pendientes y la celda estaba ocupada se incrementa el tiempo de servicio
			incrementarTiempoDeServicio(idBus-1);
			return (idBus==1? CA1_NO_PUEDE_INSERTAR_DATOS: CA2_NO_PUEDE_INSERTAR_DATOS);
		}
		
		//Si el CA no tiene datos pendientes:
		return (idBus==1? CA1_NO_TIENE_DATOS_PENDIENTES:CA2_NO_TIENE_DATOS_PENDIENTES);
	}
	

	//Sobreescribe el mtodo comprobarSolicitudEnvioDePeticion de la clase abstracta Estacion
	public int comprobarSolicitudEnvioDePeticion(int idBus)
	{
		//Si el flag solicitarPet est activado el CA enviar una solicitud de envo de peticin ro arriba al CA del bus opuesto
	 	if (solicitarPet[idBus-1])
		{
			solicitarPet[idBus-1]=false;
			
			//Se activar en el CA del bus opuesto el flag enviarRC
			if(idBus==1) enviarRc[1]=true;
			else enviarRc[0]=true;
			
			int nuevoEstado=(idBus==1? CA1_SOLICITA_ENVIO_DE_PETICION: CA2_SOLICITA_ENVIO_DE_PETICION);
			estado.add(nuevoEstado);
			return nuevoEstado;
		}
		
		//Si el flag solicitarPet no estaba activado el CA no solicitar envo de peticin
		return (idBus==1? CA1_NO_SOLICITA_ENVIO_DE_PETICION: CA2_NO_SOLICITA_ENVIO_DE_PETICION);
	}
	
	//Sobreescribe el mtodo comprobarEnvioDePeticion de la clase abstracta Estacion
	public int comprobarEnvioDePeticion(int idBus)
	{	
		if(celdaSalida[idBus-1]==null) return (idBus==1? CA1_CELDA_NO_RECIBIDA: CA2_CELDA_NO_RECIBIDA);

		//Si el CA tiene peticiones pendientes de envo
		if (peticionesPendientes[idBus-1]!=0)
		{
			//Si CA tiene una o ms peticiones pendientes pero la celda ya tiene peticion, entonces no puede marcarla
			if (celdaSalida[idBus-1].getPeticion())
			{
				return (idBus==1? CA1_NO_PUEDE_ENVIAR_PETICION: CA2_NO_PUEDE_ENVIAR_PETICION);
			}
			
			//Si CA tiene una o ms peticiones pendientes y la celda no tiene peticion, entonces la marca y decrementa el contador de peticiones pendientes
			peticionesPendientes[idBus-1]=peticionesPendientes[idBus-1]-1;
			celdaSalida[idBus-1].setPeticion(true);
			int nuevoEstado=(idBus==1? CA1_ENVIA_PETICION: CA2_ENVIA_PETICION);
			return nuevoEstado;
		}
		
		//Si no tena peticiones pendientes de envo:
		return (idBus==1? CA1_NO_TIENE_PETICION_PENDIENTE: CA2_NO_TIENE_PETICION_PENDIENTE);
	}
	
	
	
	//Sobreescribe el mtodo comprobarEnvioRC de la clase abstracta Estacion
	public int comprobarEnvioRC(int idBus)
	{
		//Si el flag enviarRc est activado el CA enviar el valor de su contador de peticiones al CA del bus opuesto
		if (enviarRc[idBus-1])
		{
			enviarRc[idBus-1]=false;
			peticionesPendientes[idBus-1]=peticionesPendientes[idBus-1]+1;
			
			if (idBus==1) 
			{
				//el valor de RC se traspasa a al contador CD (CountDown) del CA opuesto, y se resetea RC.
				cuentaAtras[1]=contadorPeticiones[0]-contadorVacias[1];
				contadorPeticiones[0]=0;
				contadorVacias[1]=0;
			}
			else
			{
				//el valor de RC se traspasa a al contador CD (CountDown) del CA opuesto, y se resetea RC.
			 	cuentaAtras[0]=contadorPeticiones[1]-contadorVacias[0];
				contadorPeticiones[1]=0;
				contadorVacias[0]=0;
			}
			
			int nuevoEstado=(idBus==1? CA1_ENVIA_RC: CA2_ENVIA_RC);	
			estado.add(nuevoEstado);
			return nuevoEstado;
		}
		return (idBus==1? CA1_NO_ENVIA_RC: CA2_NO_ENVIA_RC);
		
	}
	
	//No implementa el mecanismo de borrado de celda
	public int borrarCelda(int idBus)
	{	
		return -1;		
	}
	
}
