import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.util.*;


//CLASE QUE IMPLEMENTA EL REA DE TEXTO QUE MUESTRA LA INFORMACIN TEXTUAL ACERCA DE LOS EVENTOS OCURRIDOS DURANTE LA SIMULACIN
public class BloqueTexto extends JTextPane implements DQDB_Observer
{
	private InterfazGrafica interfaz;
	private boolean[] visibilidadEstaciones;
	private int lineasActuales;
	private StyleContext estilo;
	private Style estiloRojo, estiloNegro, estiloVerde, estiloAzul, estiloSlot;
	private int offset;
	private DefaultStyledDocument doc, docEnBlanco;
	private SimpleAttributeSet  atributos;
	private String nuevaLinea = System.getProperty("line.separator");
	private int idEstacionMonitorizada;
	private int modo;
	private CargadorDeImagenes cargador;
	//MODOS DE EXPLICACIN:
	public static final int EXPLICAR_TODAS=1;
	public static final int EXPLICAR_NINGUNA=2;
	public static final int EXPLICAR_VISIBLES=3;
	public static final int EXPLICAR_SELECCIONADA=4;
	
	ImageIcon iconoUsuarioNoGenera, iconoUsuarioGenera, iconoDatosAceptados, iconoDatosRechazados, iconoCA1CeldaOcupada, iconoCA1CeldaVacia, iconoCA2CeldaOcupada, iconoCA2CeldaVacia, iconoDatosCAaUsuario, iconoInsertaDatos, iconoBorrarCelda, iconoSolicitarEnvioPeticion, iconoRC, iconoPetOk;
	
	
	//CONSTRUCTOR
	public BloqueTexto(InterfazGrafica interfaz, boolean[] visibilidadEstaciones)
	{
		super();
		
		//Generamos una nica instancia de cada icono que se emplear, de forma que por muchos iconos que insertemos no saturemos la memoria (no cargaremos una imagen cada vez, sino referencias a una misma imagen)
		cargador=cargador.getCargador();
		iconoUsuarioNoGenera=cargador.getIcono("iconoUsuarioNoGenera.gif");
		iconoUsuarioGenera=cargador.getIcono("iconoUsuarioGenera.gif");
		iconoDatosAceptados=cargador.getIcono("iconoDatosAceptados.gif");
		iconoDatosRechazados=cargador.getIcono("iconoDatosRechazados.gif");
		iconoCA1CeldaOcupada=cargador.getIcono("iconoCA1CeldaOcupada.gif");
		iconoCA1CeldaVacia=cargador.getIcono("iconoCA1CeldaVacia.gif");
		iconoCA2CeldaOcupada=cargador.getIcono("iconoCA2CeldaOcupada.gif");
		iconoCA2CeldaVacia=cargador.getIcono("iconoCA2CeldaVacia.gif");
		iconoDatosCAaUsuario=cargador.getIcono("iconoDatosCAaUsuario.gif");
		iconoInsertaDatos=cargador.getIcono("iconoInsertaDatos.gif");
		iconoBorrarCelda=cargador.getIcono("iconoBorrarCelda.gif");
		iconoSolicitarEnvioPeticion=cargador.getIcono("iconoSolicitarEnvioPeticion.gif");
		iconoRC=cargador.getIcono("iconoRC.gif");
		iconoPetOk=cargador.getIcono("iconoPetOk.gif");
	
		this.visibilidadEstaciones=visibilidadEstaciones;
		this.interfaz=interfaz;
		idEstacionMonitorizada=-1;
		modo=EXPLICAR_TODAS;
		setBackground(new Color(249,249,249));
		setEditable(false);

		//Creamos la clase StyleContext en la que nos apoyaremos
		estilo = new StyleContext();

		//Creamos los estilos
    	estiloRojo = estilo.addStyle( null,null );
	    StyleConstants.setForeground( estiloRojo,Color.red );
	    
	    estiloVerde = estilo.addStyle( null,null );
	    StyleConstants.setForeground( estiloVerde,Color.green );
	    
	    estiloAzul = estilo.addStyle( null,null );
	    StyleConstants.setForeground( estiloAzul,Color.blue );
		
		estiloSlot = estilo.addStyle( null,null );
	    StyleConstants.setForeground( estiloSlot, Color.black );
		StyleConstants.setFontSize( estiloSlot,18 );
		
		//Creamos el StyledDocument pasndole la clase StyleContext que hemos creado
		doc = new DefaultStyledDocument(estilo);
		
		//Asociamos el documento a este "JTextPane"
		setStyledDocument(doc);
	}

	//Mtodo que permite establecer el modo de explicacin
	public void setModo(int modo)
	{
		this.modo=modo;
	}
  
  	//Mtodo que permite establecer la monitorizacin de la estacin indicada (Slo se explicarn los eventos que ocurran en esta estacin)
  	public void setEstacionMonitorizada(int idEstacion)
  	{
		idEstacionMonitorizada=idEstacion;
  	}
	
	
	//Mtodo que permite cancelar la monitorizacin de la estacin actual
	public void unsetEstacionMonitorizada() 
	{
		this.idEstacionMonitorizada=-1;
	}


	//Mtodo que permite insertar una lnea de texto
	public boolean insertarTexto(String cadena, Style estilo)
	{
		try
		{
			doc.insertString(doc.getLength(), cadena, estilo);
		
		}
		catch (BadLocationException ex) 
		{
			System.out.println("ERROR: Excepcin: BadLocationException");
			ex.printStackTrace();
			return false;
		}
		
		actualizarNumeroLineas();
		return true;		
	}
	
  	//Mtodo que permite insertar un icono en este bloque
  	public void insertarIcono(Icon icono)
	{
		setCaretPosition(getStyledDocument().getLength());
		insertIcon(icono);
	}
	
	//Mtodo que incrementa el nmero de lneas en una unidad y a continuacin comprueba si se ha alcanzado el lmite preestablecido en la clase Constantes.
	public void actualizarNumeroLineas()
	{
		lineasActuales++;
		if (lineasActuales>=Constantes.CAPACIDAD_AREA_TEXTO) 
		{
			interfaz.mostrarLimiteAreaTextoExcedido();
		
		}
	}

  	//Mtodo que resetea el contenido de este bloque
 	public void resetear()
 	{
		lineasActuales=0;
  		doc=new DefaultStyledDocument();
		setDocument(doc);
  	}	
  
  	//Mtodo que copia el contenido de este bloque al portapapeles
  	public void copiarAlPortapapeles()
  	{
		if(getSelectedText()==null || getSelectedText().compareTo("")==0 || getSelectionStart()==getSelectionEnd()) selectAll();
		copy();
		setSelectionStart(0);
		setSelectionEnd(0);		
  	}
	
	//Mtodo para notificar a este bloque que resetee las variables y el texto que contiene, para preparar una nueva simulacin
	public void notificarNuevaSimulacion(int version, int numeroEstaciones,  double pgUnica, double[] arrayPg, Estacion[] estaciones)
	{
		modo=EXPLICAR_TODAS;
		idEstacionMonitorizada=-1;
		resetear();
	}

	//Mtodo que comprueba si el bloque debe mostrar informacin o no
	public boolean comprobarSiHayQueExplicar(int idEstacion)
	{
		if(modo==EXPLICAR_NINGUNA) return false;
		if(modo==EXPLICAR_SELECCIONADA && idEstacion!=idEstacionMonitorizada) return false;
		if(modo==EXPLICAR_VISIBLES && !visibilidadEstaciones[idEstacion-1]) return false;
		return true;
	}
	
	//Sobreescribe el mtodo paintComponent para que suavice el texto que contiene
	public void paintComponent(Graphics g)
	{
		if (g instanceof Graphics2D)
		{
      			Graphics2D g2 = (Graphics2D)g;
      			Map<Object, Object> rMap = new HashMap<Object, Object>();
        		Object key = RenderingHints.KEY_ANTIALIASING;
        		Object value = RenderingHints.VALUE_ANTIALIAS_ON;
        		rMap.put(key, value);
        		g2.addRenderingHints(rMap);
    	}
    	super.paintComponent(g);
		
	}
	
	
	
	//Mtodo de la interfaz DQDB_Obsever
  	public void notificarDatosGenerados(int idslot, int idevento, int idEstacion, int evento, Datos datos)
	{
		Style estilo=estiloAzul;

		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		if (datos!=null)
		{
			insertarIcono(iconoUsuarioGenera);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO genera DATOS con destino a la estacin "+datos.getDestino()+" >>> DATOS:"+datos.toString()+nuevaLinea, estilo);
		}	
		else
		{
			insertarIcono(iconoUsuarioNoGenera);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO no genera DATOS."+nuevaLinea, estilo);
		}
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarRecepcionDeDatos(int idslot, int idevento, int idEstacion, int estadoRecepcion, Datos datos)
	{
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		//Si no transmite ninguna estacion este evento no gace nada de nada. SOLUCIONAR
		if (estadoRecepcion==Estacion.CA1_ACEPTA_DATOS) 
		{
			insertarIcono(iconoDatosAceptados);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO enva los DATOS:"+datos.toString()+" al C.A. SUPERIOR y ste los acepta. Ahora el C.A. SUPERIOR tiene una TX PENDIENTE."+nuevaLinea, estiloAzul);
		}
		else if (estadoRecepcion==Estacion.CA2_ACEPTA_DATOS) 
		{
			insertarIcono(iconoDatosAceptados);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO enva los DATOS:"+datos.toString()+" al C.A. INFERIOR y ste los acepta. Ahora el C.A. INFERIOR tiene una TX PENDIENTE."+nuevaLinea, estiloAzul);
		}
		else if (estadoRecepcion==Estacion.CA1_RECHAZA_DATOS) 
		{
			insertarIcono(iconoDatosRechazados);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO enva los DATOS:"+datos.toString()+" al C.A. SUPERIOR y este los rechaza, puesto que ya tiene una TX PENDIENTE."+nuevaLinea, estiloAzul);
		}
		else if(estadoRecepcion==Estacion.CA2_RECHAZA_DATOS) 
		{
			insertarIcono(iconoDatosRechazados);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO enva los DATOS:"+datos.toString()+" al C.A. INFERIOR y este los rechaza, puesto que ya  tiene una TX PENDIENTE."+nuevaLinea, estiloAzul);
		}
	}
	

	//Mtodo de la interfaz DQDB_Obsever
	public void notificarCeldasRecibidas(int version, int idslot, int idevento, int idEstacion, Celda celdaEntradaBus1, Celda celdaSalidaBus1, Celda celdaEntradaBus2, Celda celdaSalidaBus2, boolean incrementaVacias1, boolean incrementaVacias2)
	{
		if(idEstacion==1) insertarTexto(" SLOT "+idslot+nuevaLinea, estiloSlot);
		
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		if (version==1 || version==2)
		{
			if(celdaEntradaBus1!=null)
			{
				if (celdaEntradaBus1.getOcupada())	
				{
					insertarIcono(iconoCA1CeldaOcupada);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR ha recibido una celda ocupada: "+celdaEntradaBus1+nuevaLinea, estiloAzul);
				}
				else
				{
					insertarIcono(iconoCA1CeldaVacia);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR ha recibido una celda vaca"+nuevaLinea, estiloAzul);
				}
			}

			if(celdaEntradaBus2!=null)
			{
				if (celdaEntradaBus2.getOcupada())
				{
					insertarIcono(iconoCA2CeldaOcupada);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR ha recibido una celda ocupada: "+celdaEntradaBus2+nuevaLinea, estiloAzul);
				}
				else
				{
					insertarIcono(iconoCA2CeldaVacia);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR ha recibido una celda vaca"+nuevaLinea, estiloAzul);
				}
			}
		}
		else //if(version==3 || version==4)
		{
			if(celdaEntradaBus1!=null)
			{
				if (celdaEntradaBus1.getOcupada())	
				{
					insertarIcono(iconoCA1CeldaOcupada);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR recibe una celda ocupada "+celdaEntradaBus1+".", estiloAzul);
				}
				else
				{
					insertarIcono(iconoCA1CeldaVacia);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR recibe una celda vaca.", estiloAzul);
				}
				if (celdaSalidaBus1!=null) insertarTexto(" y enva su celda interior.", estiloAzul);
				if(celdaEntradaBus1.getPeticion()) insertarTexto(" La celda recibida est marcada con peticin -> incrementa contador de peticiones.", estiloAzul);
			}
			if (celdaSalidaBus1!=null && !celdaSalidaBus1.getOcupada()) 
			{
					if(incrementaVacias1) insertarTexto(" La celda enviada est vaca -> como CD no est activo incrementa contador de vacas. ", estiloAzul);
					else insertarTexto(" La celda enviada est vaca -> como CD est activo lo decrementa. ", estiloAzul);
			}
	
			if(celdaEntradaBus1!=null) insertarTexto(nuevaLinea, estiloAzul);
		
			
			if(celdaEntradaBus2!=null)
			{
				if (celdaEntradaBus2.getOcupada())
				{
					insertarIcono(iconoCA2CeldaOcupada);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR ha recibido una celda ocupada "+celdaEntradaBus2+".", estiloAzul);
				}
				else
				{
					insertarIcono(iconoCA2CeldaVacia);
					insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR ha recibido una celda vaca.", estiloAzul);
				}
				if (celdaSalidaBus2!=null) insertarTexto(" y enva su celda interior.", estiloAzul);
				if(celdaEntradaBus2.getPeticion()) insertarTexto(" La celda recibida est marcada con peticin -> incrementa contador de peticiones.", estiloAzul);
			}
			if (celdaSalidaBus2!=null && !celdaSalidaBus2.getOcupada()) 
			{
					if(incrementaVacias2) insertarTexto(" La celda enviada est vaca -> como CD no est activo incrementa contador de vacas. ", estiloAzul);
					else insertarTexto(" La celda enviada est vaca -> como CD est activo lo decrementa. ", estiloAzul);
			}
			
			if(celdaEntradaBus2!=null) insertarTexto(nuevaLinea, estiloAzul);
		}
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarContenidoCeldas(int idslot, int idevento, int idEstacion, int estadoCelda1, int estadoCelda2)
	{

		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		if (estadoCelda1==Estacion.CA1_ENVIA_DATOS_A_USUARIO) 
		{
			insertarIcono(iconoDatosCAaUsuario);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. SUPERIOR comprueba que la celda recibida est ocupada y enva los datos al USUARIO."+nuevaLinea, estiloAzul);
		}
		if (estadoCelda2==Estacion.CA2_ENVIA_DATOS_A_USUARIO)
		{
			insertarIcono(iconoDatosCAaUsuario);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. INFERIOR comprueba que la celda recibida est ocupada y enva los datos al USUARIO."+nuevaLinea, estiloAzul);
		}

			
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarInsercionDeDatosPendientes(int idslot, int idevento, int idEstacion, int estadoDatos1, int estadoDatos2)
	{
		
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		if (estadoDatos1==Estacion.CA1_INSERTA_DATOS) 
		{
			insertarIcono(iconoInsertaDatos);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. SUPERIOR comprueba que la celda recibida est vaca. Como tiene una TX_PENDIENTE inserta los datos en la celda y la marca como ocupada."+nuevaLinea, estiloAzul);
		}
		if (estadoDatos2==Estacion.CA2_INSERTA_DATOS) 
		{
			insertarIcono(iconoInsertaDatos);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. INFERIOR comprueba que la celda recibida est vaca. Como tiene una TX_PENDIENTE inserta los datos en la celda y la marca como ocupada."+nuevaLinea, estiloAzul);
		}
		
		if(estadoDatos1==Estacion.CA1_INSERTA_DATOS_V3V4)
		{
			insertarIcono(iconoInsertaDatos);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. SUPERIOR comprueba que la celda recibida est vaca. Como tiene una TX_PENDIENTE y el contador de espera (CD) es menor o igual que cero, inserta los datos en la celda y la marca como ocupada, finalizando la transmisin pendiente."+nuevaLinea, estiloAzul);
		}
		
		if(estadoDatos1==Estacion.CA2_INSERTA_DATOS_V3V4)
		{
			insertarIcono(iconoInsertaDatos);
			insertarTexto(" ESTACIN "+idEstacion+": C.A. INFERIOR comprueba que la celda recibida est vaca. Como tiene una TX_PENDIENTE y el contador de espera (CD) es menor o igual que cero, inserta los datos en la celda y la marca como ocupada, finalizando la transmisin pendiente."+nuevaLinea, estiloAzul);
		}
			
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarBorradoCelda(int idslot, int idevento, int idEstacion, int estadoCA1, int estadoCA2)
	{
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		
		if (estadoCA1==Estacion.CA1_BORRA_DATOS) 
		{
			insertarIcono(iconoBorrarCelda);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO comprueba que es el destinatario de los DATOS recibidos, por lo que enva una seal de borrado al C.A. SUPERIOR."+nuevaLinea, estiloAzul);
		}
		if(estadoCA2==Estacion.CA2_BORRA_DATOS) 
		{
			insertarIcono(iconoBorrarCelda);
			insertarTexto(" ESTACIN "+idEstacion+": USUARIO comprueba que es el destinatario de los DATOS recibidos, por lo que enva una seal de borrado al C.A. INFERIOR."+nuevaLinea, estiloAzul);
		}
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarSolicitudEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		if(estadoPeticion1==Estacion.CA1_SOLICITA_ENVIO_DE_PETICION) 
		{
			insertarIcono(iconoSolicitarEnvioPeticion);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR solicita al C.A.INFERIOR el envo de una peticin."+nuevaLinea, estiloAzul);
		}
		
		if(estadoPeticion2==Estacion.CA2_SOLICITA_ENVIO_DE_PETICION) 
		{
			insertarIcono(iconoSolicitarEnvioPeticion);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR solicita al C.A.SUPERIOR el envo de una peticin."+nuevaLinea, estiloAzul);
		}
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarEnvioRC(int idslot, int idevento, int idEstacion, int estadoRC1, int estadoRC2, int[] contadorPeticiones, int[] contadorVacias, int[] cuentaAtras)
	{
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		if(estadoRC1==Estacion.CA1_ENVIA_RC) 
		{	
			
			insertarIcono(iconoRC);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR enva el valor de su contador de peticiones al C.A.INFERIOR. El C.A.INFERIOR CALCULA EL VALOR DE CD:  CD=PETICIONES(CASup)-VACIAS(CAInf) = "+cuentaAtras[1]+nuevaLinea, estiloAzul);		
		}
		if(estadoRC2==Estacion.CA2_ENVIA_RC) 
		{
			insertarIcono(iconoRC);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR enva el valor de su contador de peticiones al C.A.SUPERIOR. El C.A.SUPERIOR CALCULA EL VALOR DE CD:  CD=PETICIONES(CAInf)-VACIAS(CASup)= "+cuentaAtras[0]+nuevaLinea, estiloAzul);
		}
	}
	
	//Mtodo de la interfaz DQDB_Obsever
	public void notificarEnvioDePeticion(int idslot, int idevento, int idEstacion, int estadoPeticion1, int estadoPeticion2)
	{
		if(!comprobarSiHayQueExplicar(idEstacion)) return;
		if(estadoPeticion1==Estacion.CA1_ENVIA_PETICION) 
		{
			insertarIcono(iconoPetOk);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.SUPERIOR comprueba que el contador de peticiones pendientes de envo es mayor que cero y que la celda recibida tiene el campo peticin libre, as que la marca con peticin y decrementa su contador de peticiones pendientes."+nuevaLinea, estiloAzul);
		}
		
		if(estadoPeticion2==Estacion.CA2_ENVIA_PETICION) 
		{
			insertarIcono(iconoPetOk);
			insertarTexto(" ESTACIN "+idEstacion+": C.A.INFERIOR comprueba que el contador de peticiones pendientes de envo es mayor que cero y que la celda recibida tiene el campo peticin libre, as que la marca con peticin y decrementa su contador de peticiones pendientes."+nuevaLinea, estiloAzul);
		}
	}
	
	
}