import java.awt.*;
import java.util.*;
import javax.swing.*;

//CLASE QUE IMPLEMENTA LAS GRFICAS MLTIPLES
public class GraficaMultiple extends Grafica {

	private int marcadorDeNavegacion=0;
	private int idBus;
	private Vector <Vector<Punto>> lista;
	private GraficaMultiple graficaZoom;
	
	//CONSTRUCTOR 1
	public GraficaMultiple(Image background, Image labelX, Image labelY, Image titulo, int idEstacion, BloqueEstadisticas listener, int numEstaciones, int idBus)
	{
		super(background, labelX, labelY, titulo, idEstacion, listener);

		lista=new Vector <Vector<Punto>>();
		Vector<Punto> puntos=new Vector<Punto>();
			
		this.idBus=idBus;
			
		Dimension dim=new Dimension(background.getWidth(this),background.getHeight(this)+margenY);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);
			
	
		Vector<Punto> vector;
		for(int i=0; i<numEstaciones; i++)
		{
			vector=new Vector<Punto>();
			lista.add(vector);
		}
	}




	//CONSTRUCTOR 2
	public GraficaMultiple(Image background, Image labelX, Image labelY, Image titulo, int idEstacion, Vector<Vector<Punto>> lista, int separadores, double xmax, double ymax, int marcadorDeNavegacion,  BloqueEstadisticas listener)
	{
		super(background, labelX, labelY, titulo, idEstacion, lista.elementAt(0), separadores, xmax, ymax, marcadorDeNavegacion, listener);
		this.lista=lista;
		this.marcadorDeNavegacion=marcadorDeNavegacion;
	}


	//Mtodo que inserta un punto en la grfica
	public void insertarPunto(double x, double y, int idGrafica)
	{

		if(idGrafica==0) marcadorDeNavegacion++;
		if(x>xmax)xmax=x;
		if(y>ymax)ymax=y;

		lista.elementAt(idGrafica).addElement(new Punto(x, y));
		
		if (graficaZoom!=null)
		{
			graficaZoom.updateMaximos(xmax, ymax);
			if (idGrafica==0) 	
				graficaZoom.incrementarMarcadorDeNavegacion();
		
		}
		repaint();
	}
	

	//Incrementa en una unidad el marcador de navegacin
	public boolean incrementarMarcadorDeNavegacion()
	{
		if (marcadorDeNavegacion>=lista.elementAt(0).size()) return false;

		marcadorDeNavegacion++;
		actualizarMaximos();
		repaint();
		return true;
	}

	//Decrementa en una unidad el marcador de navegacin
	public boolean decrementarMarcadorDeNavegacion()
	{
		if(marcadorDeNavegacion==0) return false;
		marcadorDeNavegacion--;
		actualizarMaximos();
		return false;
	}


	//Mtodo que calcula las coordenadas x mxima e y mxima en el rango comprendido entre "0 y marcadorDeNavegacin", necesarias para poder establecer la escala de los ejes y dicujar los puntos
	public void actualizarMaximos()
	{
		double nuevaXmax=0;
		double nuevaYmax=0;
		Punto punto;
		Vector<Punto> puntos;
		
		for(int j=0; j<lista.size(); j++)
		{
			puntos=lista.elementAt(j);
			int imax=Math.min(marcadorDeNavegacion, puntos.size());
			for(int i=0; i<imax; i++)
			{
				punto=puntos.elementAt(i);
				if(nuevaXmax<punto.getX()) nuevaXmax=punto.getX();
				if(nuevaYmax<punto.getY()) nuevaYmax=punto.getY();
		
			}
		}
		
		xmax=nuevaXmax;
		ymax=nuevaYmax;
		repaint();
	}
	

	//Mtodo que genera una rplica de esta grfica pero de mayor tamao
	public GraficaMultiple zoom()
	{
		return graficaZoom=new GraficaMultiple(CargadorDeImagenes.getCargador().getImagen("graficaZoom.gif"), labelX, labelY, titulo, idEstacion, lista, 20, xmax, ymax, marcadorDeNavegacion, listener);
	}
	
	
	//Mtodo encargado de pintar la grfica
	public void paintComponent( Graphics g )
	{
		super.paintComponent(g);

		//Dibuja el fondo de la grfica	
		g.drawImage(background, 0, margenY, this);		
		
		//Dibuja el ttulo de la grfica y los nombres de los ejes
		g.drawImage(titulo, (background.getWidth(this)-titulo.getWidth(this))/2, margenY+7, this);
		g.drawImage(labelX, (background.getWidth(this)-labelX.getWidth(this))/2, margenY+background.getHeight(this)-20, this);
		g.drawImage(labelY,10, margenY+(background.getHeight(this)-labelY.getHeight(this))/2, this);
		
		//Dibuja el rea de la grfica que contendr los puntos
		g.setColor(Color.white);
		g.fillRect(origenX, origenY-TamEjeY,TamEjeX,TamEjeY);
		g.setColor(Color.lightGray);
		g.drawLine(origenX+TamEjeX, origenY-TamEjeY, origenX+TamEjeX,origenY);
		g.drawLine(origenX, origenY-TamEjeY, origenX+TamEjeX,origenY-TamEjeY);	

	



		Vector<Punto> puntos;
		for(int j=0; j<lista.size(); j++)
		{
			//Pintamos el nmero de estacin que corresponde al cuadro de "leyenda"
			g.setColor(Color.black);
			if(idBus==1) g.drawString("EST."+(j+1), background.getWidth(this)-41, margenY+49+j*20);
			else g.drawString("EST."+(j+2), background.getWidth(this)-41, margenY+49+j*20);
			
			//Seleccionamos el color que corresponde para pinar el cuadro de "leyenda"
			if(j==0) g.setColor(Color.blue);
			else if(j==1) g.setColor(Color.red);
			else if(j==2) g.setColor(Color.green);
			else if(j==3) g.setColor(Color.orange);
			else if(j==4) g.setColor(Color.black);
			else if(j==5) g.setColor(new Color(153, 0, 204));
			else if(j==6) g.setColor(new Color(255, 92, 174));
			else if(j==7) g.setColor(new Color(102,153,255));
			else if(j==8) g.setColor(new Color(255, 238, 0));
		
			//Pintamos el cuadro de "leyenda"
			if(esZoom) g.fillRect(background.getWidth(this)-49, margenY+40+j*20, 8, 10);
			else g.fillRect(background.getWidth(this)-57, margenY+40+j*20, 15, 10);
			
			//Pintamos el marco del cuadro de "leyenda"
			Color colorPuntos=g.getColor();
			g.setColor(Color.black);
			if(esZoom) g.drawRect(background.getWidth(this)-49, margenY+40+j*20, 7, 10);
			else g.drawRect(background.getWidth(this)-57, margenY+40+j*20, 14, 10);
			
			//Pintamos los puntos
			g.setColor(colorPuntos);
			puntos=lista.elementAt(j);
			int imax=Math.min(puntos.size(), marcadorDeNavegacion);
			for(int i=0;i<imax;i++)
			{	
				//Pinta los puntos
				g.fillOval(origenX+(int)Math.round(TamEjeX*puntos.elementAt(i).getX()/xmax)-2, 
					origenY-(int)Math.round(TamEjeY*puntos.elementAt(i).getY()/ymax)-2,5,5);  
			}	
		}

		
		//Dibuja los ejes
		g.setColor(Color.black);
		g.drawLine(origenX, origenY-TamEjeY, origenX,origenY);
		g.drawLine(origenX, origenY, origenX+TamEjeX,origenY);
		
		//Pinta los separadores de los ejes y sus valores
		g.setColor(Color.black);
		g.setFont(new Font("Verdana", Font.PLAIN, 9));
		for(int i=1;i<=separadores;i++)
		{
			//pinta separadores de ejes
			g.drawLine(origenX-3, origenY-(int)Math.round(i*TamEjeY/separadores) , origenX+2, origenY-(int)Math.round(i*TamEjeY/separadores) );	
			g.drawLine(origenX+(int)Math.round(i*TamEjeX/separadores), origenY-2 , origenX+(int)Math.round(i*TamEjeX/separadores), origenY+3 );

			//pinta los valores de los separadores de los ejes
			if (ymax>=10)
				g.drawString( ""+ (Math.round(i*ymax*100/separadores)/100.0),  origenX-35,  origenY-(int)Math.round(i*TamEjeY/separadores) +5);
			else 
				g.drawString( ""+ (Math.round(i*ymax*1000/separadores)/1000.0),  origenX-35,  origenY-(int)Math.round(i*TamEjeY/separadores) +5);			
		
				if ((xmax>0))	
			

			if (xmax>=10)
				g.drawString( ""+ (Math.round(i*xmax*100/separadores)/100.0),  origenX+(int)Math.round(1*i*TamEjeX/separadores)-8, origenY+13 );
			else
				g.drawString( ""+ (Math.round(i*xmax*1000/separadores)/1000.0),  origenX+(int)Math.round(1*i*TamEjeX/separadores)-8, origenY+13 );	
		
		
		}
		
	}

}




