import javax.swing.ImageIcon;
import java.awt.Image;
import java.awt.*;
import java.net.*;

//CLASE QUE CENTRALIZA LA CARGA DE IMGENES, DE FORMA QUE SE PUEDA CAMBIAR EL MTODO SEGN LAS NECESIDADES.
// Por ejemplo: Hacer que se carguen las imgenes desde el interior de un jar requiere cargar las imgenes de una forma especial, diferente al modo normal.
// Al tener el proceso de carga de imgenes centralizado, si queremos modificar el mtodo de carga de imgenes, slo tendremos que modificar esta clase, sin tener que hacer modificaciones en todas las clases
public class CargadorDeImagenes
{
	private static CargadorDeImagenes cargador;
	
	//CONSTRUCTOR
	private CargadorDeImagenes(){}
	
	//Mtodo esttico que devuelve el objeto cargador. Todas las clases usan el mismo objeto cargador de imgenes, no hay mltiples objetos de esta clase.
	public static CargadorDeImagenes getCargador()
	{
		if(cargador==null) cargador=new CargadorDeImagenes();
		return cargador;
	}
	
	//Devuelve la imagen cuyo nombre se especifica en el parmetro de entrada
	public Image getImagen(String imagen)
	{	
		return new ImageIcon(getClass().getClassLoader().getResource(Constantes.directorioImagenes+imagen)).getImage();
	}
	
	//Devuelve el icono cuyo nombre se especifica en el parmetro de entrada
	public ImageIcon getIcono(String icono)
	{
		return new ImageIcon(getClass().getClassLoader().getResource(Constantes.directorioImagenes+icono));
	}
	
	//Devuelve el cursor cuyo nombre se especifica en el parmetro de entrada
	public Cursor getCursor(String nombreCursor)
	{	
		Toolkit herramienta=Toolkit.getDefaultToolkit();
		Cursor cursor=herramienta.createCustomCursor(herramienta.getImage(getClass().getClassLoader().getResource(Constantes.directorioImagenes+nombreCursor)),new Point(0,0),"cursor"); //CAMBIAR PUNTERO A CURSOR POR DEFECTO
		return cursor;
	}
	
	//Devuelve el URL asociado al recurso especificado
	public String getURL(String recurso)
	{
		return getClass().getClassLoader().getResource(recurso).toString();
	}
}